from __future__ import annotations

from tcsoa.gen.StructureManagement._2012_09.Structure import AddParam, AddResponse, ParentChildPair
from tcsoa.gen.BusinessObjects import BOMLine, ItemRevision
from tcsoa.gen.StructureManagement._2012_09.VariantManagement import CreateAndSubsVIResponse, CreateAndSubsVIInput, CreateVIInput, CreateVIResponse
from typing import List
from tcsoa.gen.StructureManagement._2012_09.MassUpdate import MassUpdateExecuteECRresponse, MassUpdateAffectedResponse, MassUpdateExecuteECRinput, MassUpdateExecuteECNresponse, MassUpdateAffectedInput
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class MassUpdateService(TcService):

    @classmethod
    def massGetAffectedParents(cls, input: List[MassUpdateAffectedInput]) -> MassUpdateAffectedResponse:
        """
        This operation will call ITK PS__masschange_get_parents to get all the affected impacted parent parts and there
        selectablility for mass update.
        """
        return cls.execute_soa_method(
            method_name='massGetAffectedParents',
            library='StructureManagement',
            service_date='2012_09',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=MassUpdateAffectedResponse,
        )

    @classmethod
    def massUpdateExecutionECN(cls, changeItemRevs: List[ItemRevision]) -> MassUpdateExecuteECNresponse:
        """
        This operation will call ITK PS__masschange_execute which will process all the change item revision markup
        changes recorded during the ECR CM process.
        """
        return cls.execute_soa_method(
            method_name='massUpdateExecutionECN',
            library='StructureManagement',
            service_date='2012_09',
            service_name='MassUpdate',
            params={'changeItemRevs': changeItemRevs},
            response_cls=MassUpdateExecuteECNresponse,
        )

    @classmethod
    def massUpdateExecutionECR(cls, input: List[MassUpdateExecuteECRinput]) -> MassUpdateExecuteECRresponse:
        """
        This operation will call one of three ITKs: execute mode=1 calls PS__PS__masschange_onetime, execute mode=2
        calls PS__masschange_authorize_add and execute mode=3 calls PS__masschange_authorize_remove.
        """
        return cls.execute_soa_method(
            method_name='massUpdateExecutionECR',
            library='StructureManagement',
            service_date='2012_09',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=MassUpdateExecuteECRresponse,
        )


class VariantManagementService(TcService):

    @classmethod
    def createAndSubstituteVariantItem(cls, createAndSubstituteVIInput: List[CreateAndSubsVIInput]) -> CreateAndSubsVIResponse:
        """
        This operation will create new variant Item for given BOMLine (also called as 'Generic BOMLine') from a BOM
        structure (also called as 'Generic Structure') having variability using variants. Addition to creating new
        variant Item, this is operation will also replace or substitute newly created variant Item Revision in given
        target BOMLine (also called as 'VI BOMLine') in variant Structure which corresponding to fully configured
        structure by fixing variability in Generic BOM Structure.
        
        Operation also accepts 2 flags ''findVIBeforeCreate'' used to control if existing variant Item should be
        searched and used instead of creating new variant Item and ''linkVIToGenericItem'' to link newly created
        variant Item to source Item of generic BOMLine.
        
        The new variant Item can be created in 2 ways either creating new separate Item or doing "Save-As" operation on
        generic Item. In case of "Save-As" the parameter ''CreateOrSaveAsDescriptor'' will provide additional
        information about which all related objects are carried over to new Item from source generic Item.
        
        
        Use cases:
        This operation should be used when user has Generic Structure & corresponding created variant Structure and
        user wants to create Item which is variant for each child BOMLine object and replace in variant Structure.
        """
        return cls.execute_soa_method(
            method_name='createAndSubstituteVariantItem',
            library='StructureManagement',
            service_date='2012_09',
            service_name='VariantManagement',
            params={'createAndSubstituteVIInput': createAndSubstituteVIInput},
            response_cls=CreateAndSubsVIResponse,
        )

    @classmethod
    def createVariantItem(cls, createVIInputs: List[CreateVIInput]) -> CreateVIResponse:
        """
        This operation will create new variant Item for given BOMLine (also called as 'Generic BOMLine') from a BOM
        structure (also called as 'Generic Structure') having variability using variant Options.
        
        Operation also accepts a flag ''linkVIToGenericItem'' to link newly created variant Item to source Item of
        'generic BOMLine'.
        
        The new variant Item can be created in 2 ways either by creating new separate Item or doing "Save-As" operation
        on generic Item. In case of "Save-As" the parameter 
        ''CreateOrSaveAsDescriptor'' will provide additional information about which all related objects are carried
        over to new Item from source generic Item.
        
        
        Use cases:
        This operation should be used when user wants to create new variant Item using generic BOMLine from a generic
        BOM structure.
        """
        return cls.execute_soa_method(
            method_name='createVariantItem',
            library='StructureManagement',
            service_date='2012_09',
            service_name='VariantManagement',
            params={'createVIInputs': createVIInputs},
            response_cls=CreateVIResponse,
        )


class StructureService(TcService):

    @classmethod
    def validateOccurrenceConditions(cls, lines: List[BOMLine], flag: int) -> ServiceData:
        """
        The operation is to validate occurrences of specified lines and their substitutes against occurrence conditions
        with option to validate the whole substructure.
        
        Use cases:
        User imports a structure which has some substitutes and wants to validate the structure against occurrence
        conditions.  Invoke the operation by passing in the root line and the flag for recursive validation. Failed
        BOMLine validations are returned in the 'ServiceData' object.
        """
        return cls.execute_soa_method(
            method_name='validateOccurrenceConditions',
            library='StructureManagement',
            service_date='2012_09',
            service_name='Structure',
            params={'lines': lines, 'flag': flag},
            response_cls=ServiceData,
        )

    @classmethod
    def validateParentChildConditions(cls, input: List[ParentChildPair]) -> ServiceData:
        """
        The operation is to validate parent and child objects against occurrence conditions.
        
        Use cases:
        User invokes the operation to validate against occurrence conditions before creating two occurrences by using
        an ItemRevision as parent for the two occurrences and a General Design Element object as one child and another
        ItemRevision as another child.
        """
        return cls.execute_soa_method(
            method_name='validateParentChildConditions',
            library='StructureManagement',
            service_date='2012_09',
            service_name='Structure',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def add(cls, input: List[AddParam]) -> AddResponse:
        """
        The operation adds business objects as child lines or substitutes of the specified lines with option to
        propagate transform matrix.
        .  The business objects can be item, item revision, General Design Elements, ImanItemLines or GDELines
        . If the business object to be added is a pending cut line, then the pending cut line will be processed after
        it is added.
        . If the business object is a WorkArea object or a line of WorkArea object and the object is to be added to
        WorkArea structure, then it will be added with predecessor relation.
        . If the object to be added is a line that contains Incremental Change Elements, the elements will be carried
        over to the newly created line.
        . BOMLine property values can be specified for the new line.
        . Occurrence type can be specified for the newline as one BOMLine property but will be handled specially.
        
        Use cases:
        - User  wants to add an item to a line. He/she invokes the operation to add it with initial values for find 
        number, quantity, etc. The line will be created with the initial BOMLine properties.
        - User wants to add an item revision as a substitute of a precise line. He/she invokes the operation to add it.
        - User invokes the operation to copy and paste a line. The Incremental Change Elements are carried over to the
        new line.
        - User invokes the operation to cut and paste a GDE line.
        - User paste a WorkArea object to a WorkArea structure, the newly added line is also added with the predecessor
        relationship.
        
        """
        return cls.execute_soa_method(
            method_name='add',
            library='StructureManagement',
            service_date='2012_09',
            service_name='Structure',
            params={'input': input},
            response_cls=AddResponse,
        )


class PublishByLinkService(TcService):

    @classmethod
    def deleteLinksForSource2(cls, sourceBOMLines: List[BOMLine], dataToUnpublish: str) -> ServiceData:
        """
        Finds and deletes PublishLink for input source BOMLine objects. 
        
        The AbsOccXform of the target BOMLine objects will be deleted if 'dataToUnpublish' is TRANSFORM. All in context
        JTs of the target BOMLine objects will be unattached if 'dataToUnpublish' is SHAPE. If all data(as of now
        TRANSFORM and SHAPE only) needs to be removed then value of 'dataToUnpublish' should be ALL. None of the
        published data will be impacted when 'dataToPublish' is empty string.
        
        Use cases:
        Delete PublishLink and unpublish data from target BOMLine.
        """
        return cls.execute_soa_method(
            method_name='deleteLinksForSource2',
            library='StructureManagement',
            service_date='2012_09',
            service_name='PublishByLink',
            params={'sourceBOMLines': sourceBOMLines, 'dataToUnpublish': dataToUnpublish},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awb0Element, Awb0ProductContextInfo
from tcsoa.gen.Internal.ActiveWorkspaceBom._2012_10.OccurrenceManagement import InfoForElementCreation
from typing import Dict, List
from tcsoa.gen.Internal.ActiveWorkspaceBom._2019_06.OccurrenceManagement import NewElementInfo3
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_05.OccurrenceManagement import OccurrenceSortInput
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_12.OccurrenceManagement import OccurrenceInfo6


@dataclass
class OccTypeInfo(TcBaseObj):
    """
    Contains  parent, map/list of OccTypeName objects retrieved based on parent, source and request preference.
    
    :var parentObject: The parent of selected object. Valid types are Item, ItemRevision, BOMLine or Awb0Element.
    :var srcObjectOccTypesMap: A map (BusinessObject, list of OccTypeName) of source BusinessObject to list of
    OccTypeName. When preference "useMEDisplayOccurrenceTypePref" with value true is specified the operation shall
    return a map containing entries. An empty map is returned otherwise.
    :var srcObjectDefaultOccTypeMap: A map (BusinessObject, OccTypeName) of source BusinessObject to default
    OccTypeName. The default occurrence type is decided based on Teamcenter preference
    MEAssignCustomizedOccurrenceType. When preference "useMEDisplayOccurrenceTypePref" with value true is specified the
    operation shall return a map containing entries. An empty map is returned otherwise.
    :var occTypeNames: A list of OccTypeName objects when value of preference "useMEDisplayOccurrenceTypePref" is not
    set or is false.
    """
    parentObject: BusinessObject = None
    srcObjectOccTypesMap: SourceObjectOccTypesMap = None
    srcObjectDefaultOccTypeMap: SourceObjectDefaultOccTypeMap = None
    occTypeNames: List[OccTypeName] = ()


@dataclass
class OccTypeName(TcBaseObj):
    """
    Contains internal and display name of the allowed PSOccurrenceType.
    
    :var internalName: Internal name of the PSOccurrenceType
    :var displayName: Display name of the PSOccurrenceType
    """
    internalName: str = ''
    displayName: str = ''


@dataclass
class OccTypesInputData(TcBaseObj):
    """
    Contains parent and list of source objects. Optionally request contains criteria to filter occurrence types based
    on Teamcenter preference MEDisplayOccurrenceType.
    
    :var parentObject: The parent of selected object.Valid types are Item, ItemRevision, BOMLine or Awb0Element. This
    field is ignored if request preference useMEDisplayOccurrenceTypePref is false.
    :var sourceObjects: A list of BusinessObject for which allowed occurrence type information is retrieved.Valid types
    are Item, ItemRevision, BOMLine or Awb0Element. This field is ignored if request preference
    useMEDisplayOccurrenceTypePref is false.
    :var requestPref: A map (string, string) of preference names and value pairs. Allowed preference name is 
    useMEDisplayOccurrenceTypePref. Allowed values are true/false. Keys and values are case sensitive.
    """
    parentObject: BusinessObject = None
    sourceObjects: List[BusinessObject] = ()
    requestPref: RequestPreference12 = None


@dataclass
class OccTypesResp(TcBaseObj):
    """
    The PSOccurrenceType internal and display names.
    
    :var occTypeInfo: Contains  parent and list/map of OccTypeName objects.
    :var serviceData: The Service Data with partial errors for getAllowedOccurrenceTypes operation.
    """
    occTypeInfo: OccTypeInfo = None
    serviceData: ServiceData = None


@dataclass
class RemoveLevelInputData(TcBaseObj):
    """
    A list of selected Awb0Element objects which are to be removed.
    
    :var elementsToRemoveLevel: A list of Awb0Element objects to be removed from content and their children to be added
    as children of parent of these objects.
    :var productContext: Awb0ProductContextInfo contains the product as well as all the configuration information.
    :var sortCriteria: Criteria to sort the occurrences based on internal property name and sorting order e.g. "ASC"
    and "DESC".
    """
    elementsToRemoveLevel: List[Awb0Element] = ()
    productContext: Awb0ProductContextInfo = None
    sortCriteria: OccurrenceSortInput = None


@dataclass
class ResetWorkingContextInputData(TcBaseObj):
    """
    Contains the Awb0ProductContextInfo for the opened object and a map of preference names and value pairs.
    
    :var productContext: Awb0ProductContextInfo containing configuration information.
    :var requestPref: A map (string, string) of preference names and value pairs. Allowed preference names are:
    "deleteCloneData". Allowed values are: True/False.  Keys and values are case sensitive. When preference
    "deleteCloneData" with value true is specified the operation shall delete clone data associated with
    Awb0Autobookmark. If map is empty then special processing like &lsquo;deleteCloneData&rsquo; is not be performed.
    """
    productContext: Awb0ProductContextInfo = None
    requestPref: RequestPreference12 = None


@dataclass
class ResetWorkingContextResp(TcBaseObj):
    """
    Contains ServiceData and map of preference names and value pairs.
    
    :var serviceData: The Service Data.
    :var responsePref: A map (string, string) of preference names and value pairs. Allowed preference names are:
    "recipeReset". Allowed values are: True/False. Keys and values are case sensitive. If filterRecipeList assoicated
    with Awb0Autobookmark has been cleared then preference "recipeReset" is set to true.
    """
    serviceData: ServiceData = None
    responsePref: RequestPreference12 = None


@dataclass
class InfoForInsertLevelData(TcBaseObj):
    """
    A list of selected Awb0Element objects for which new element would be added as a parent.
    
    :var elementsToBeReparented: A list of Awb0Element objects to be reparented.
    :var fetchAllowedOccRevTypes: If true, operation returns allowed occurrence revision (Fnd0AbstractOccRevision)
    types.The allowed occurrence revision types can be specified using preference TcAllowedOccRevTypes_<ParentItemType>.
    """
    elementsToBeReparented: List[Awb0Element] = ()
    fetchAllowedOccRevTypes: bool = False


@dataclass
class InfoForInsertLevelResp(TcBaseObj):
    """
    The allowable parent type name and information for Full Text Search along with preferred object type information.
    
    :var preferredTypeInfo: Preferred type information to create new parent Awb0Element.
    :var allowedTypeInfos: A list of all allowed types to create Awb0Element as parent of the selected Awb0Element
    objects.
    :var serviceData: The ServiceData containing partial error if any.
    """
    preferredTypeInfo: InfoForElementCreation = None
    allowedTypeInfos: List[InfoForElementCreation] = ()
    serviceData: ServiceData = None


@dataclass
class InsertLevelInputData2(TcBaseObj):
    """
    The information to insert a BusinessObject as parent of the given Awb0Element objects.
    
    :var elements: A list of Awb0Element objects which the user intends to group and create a new parent for.
    :var objectToBeInserted: Business object for which a Awb0Element is to be created and inserted as parent of the
    given Awb0Element objects. The supported types are Item, ItemRevision, and Awb0Element.
    :var productContext: Awb0ProductContextInfo contains the product as well as all the configuration information.
    :var sortCriteria: Criteria to sort the occurrences based on internal property name and sorting order e.g. "ASC"
    and "DESC".
    """
    elements: List[Awb0Element] = ()
    objectToBeInserted: BusinessObject = None
    productContext: Awb0ProductContextInfo = None
    sortCriteria: OccurrenceSortInput = None


@dataclass
class InsertLevelResponse2(TcBaseObj):
    """
    InsertLevelResponse2 structure contains successfully inserted Awb0Element object corresponding to the input and the
    ServiceData
    
    :var newParent: The Awb0Element object which is the new inserted parent object of the given set of objects.
    :var childOccurrencesInfo: The map(Awb0Element, std::vector<OccurrenceInfo6>) of parent Awb0Element object
    (inserted and old parent) to the list of OccurrenceInfo6 corresponds to the child occurrrences.
    :var newElementInfos: A list of NewElementInfo3 populated when the parent is instantiated more than once.
    :var serviceData: The service data containing partial error if any.
    """
    newParent: Awb0Element = None
    childOccurrencesInfo: ParentToChildOccurrencesMap = None
    newElementInfos: List[NewElementInfo3] = ()
    serviceData: ServiceData = None


@dataclass
class RemoveLevelResponse(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    pass


"""
The map which can have a key and value pair. The key and value are case sensitive.
"""
RequestPreference12 = Dict[str, List[str]]


"""
A map (BusinessObject/OccTypeName) of source BusinessObject and default OccTypeName object.
"""
SourceObjectDefaultOccTypeMap = Dict[BusinessObject, OccTypeName]


"""
A map (BusinessObject/list of OccTypeName) of source BusinessObject and  list of OccTypeName objects.
"""
SourceObjectOccTypesMap = Dict[BusinessObject, List[OccTypeName]]


"""
The map(Awb0Element, std::vector<OccurrenceInfo6>) of parent Awb0Element object to the list of OccurrenceInfo6 corresponds to the child occurrrences.
"""
ParentToChildOccurrencesMap = Dict[Awb0Element, List[OccurrenceInfo6]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AllocatedTime(TcBaseObj):
    """
    Allocated Time structure for allocatedTimeRollUp operation
    
    :var roots: vector of Tags of the item element
    :var calculatedBy: The type of calculation for the allocated time algorythm.possible values:
    duration_time,simulated_time,estimated_time,best_available_time.
    """
    roots: List[BusinessObject] = ()
    calculatedBy: str = ''


@dataclass
class AllocatedTimeResponse(TcBaseObj):
    """
    Return structure for allocatedTimeRollUp operation
    
    :var results: MAp of each root and its allocated time result.
    :var serviceData: This is a common data strucuture used to return sets of
    Teamcenter Data Model object from a service request. This also
    holds services exceptions.
    """
    results: MapAllocatedTimeResults = None
    serviceData: ServiceData = None


@dataclass
class TimeAnalysisInputs(TcBaseObj):
    """
    Time Analysis Inputs structure for timeAnalysisRollup operation
    
    :var roots: vector of Tags of the item element
    :var runTimePropertiesToRecalc: The additional run time  properties need to be calculated for the time analysis
    such as total time and duration time
    """
    roots: List[BusinessObject] = ()
    runTimePropertiesToRecalc: List[str] = ()


@dataclass
class TimeAnalysisResult(TcBaseObj):
    """
    Time Analysis Results structure for timeAnalysisRollup operation
    
    :var object: the item element for the rollup.
    :var categoryResults: a map results of the activity total time for each category.
    :var mapRunTimeResult: a map results of the requested run time properties.
    """
    object: BusinessObject = None
    categoryResults: MapStringDouble = None
    mapRunTimeResult: MapStringDouble = None


@dataclass
class TimeAnalysisRollupResponse(TcBaseObj):
    """
    Return structure for timeAnalysisRollup operation
    
    :var results: vector of the TimeAnalysisResult.
    :var serviceData: This is a common data strucuture used to return sets of
    Teamcenter Data Model object from a service request. This also
    holds services exceptions.
    """
    results: List[TimeAnalysisResult] = ()
    serviceData: ServiceData = None


"""
MapAllocatedTimeResults
"""
MapAllocatedTimeResults = Dict[BusinessObject, float]


"""
MapStringDouble
"""
MapStringDouble = Dict[str, float]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Fnd0ProxyTask, ScheduleTask, ResourceAssignment, TaskDependency
from tcsoa.gen.Internal.ProjectManagement._2008_06.ScheduleManagement import GenericAttributesContainer
from typing import List
from tcsoa.gen.ProjectManagement._2011_06.ScheduleManagement import SchMgtOptions
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LoadProgramViewContainer(TcBaseObj):
    """
    An input container containing the list of schedules and filter conditions.
    
    :var schedules: The UIDs of the Schedules which are included in this program view.
    :var loadProgramViewSets: The collection of filters to be applied.
    """
    schedules: List[str] = ()
    loadProgramViewSets: List[LoadProgramViewSet] = ()


@dataclass
class LoadProgramViewFilter(TcBaseObj):
    """
    The represents the a single input filter in the query for loading the program view.
    
    :var field: The integer value of property to filter out tasks or schedules.
    :var value: The value of the property to filter out schedules or tasks.
    :var condition: The condition: greater-than, less-than etc.
    """
    field: str = ''
    value: str = ''
    condition: int = 0


@dataclass
class LoadProgramViewSet(TcBaseObj):
    """
    The collection of input filters.
    
    :var loadProgramViewFilters: The filters.
    """
    loadProgramViewFilters: List[LoadProgramViewFilter] = ()


@dataclass
class ScheduleModifyContainer(TcBaseObj):
    """
    The information to update a schedule.  It contains new, updated, and deleted tasks, resource assignments, and task
    dependencies information for a schedule.
    
    :var scheduleUid: the schedule internal name (uid)
    :var scheduleUpdates: the schedule updates- GenericAttributesContainer{ object-schedule object
    tag,keyValContainer(can be empty),StringValContainer(can be  empty),typedAttributeContainer(can be empty)  }
    :var deletedDependencies: The deleted dependencies
    :var deletedAssignments: The deleted assignments
    :var newProxyTasks: The information to create new ProxyTask in this schedule
    :var proxyTaskUpdates: A list of known updated proxy tasks.
    :var deletedProxyTasks: A list of proxy tasks to be deleted.
    :var insertedSubSchedules: inserted schedules
    :var detachedSubSchedules: detatched sub schedules
    :var options: Schedule modification options
    1) logical Options (can be empty)
    2) Integer Options (can be empty)
    3) String Options (can be empty)
    :var newTasks: The new tasks to add
    :var newDependencies: The new dependencies
    :var newAssignments: The new assignments
    :var taskUpdates: The existing tasks update
    :var dependencyUpdates: The existing dependencies updates
    :var assignmentUpdates: The existing assignments update
    :var deletedTasks: The deleted tasks
    """
    scheduleUid: str = ''
    scheduleUpdates: GenericAttributesContainer = None
    deletedDependencies: List[TaskDependency] = ()
    deletedAssignments: List[ResourceAssignment] = ()
    newProxyTasks: List[GenericAttributesContainer] = ()
    proxyTaskUpdates: List[GenericAttributesContainer] = ()
    deletedProxyTasks: List[BusinessObject] = ()
    insertedSubSchedules: List[GenericAttributesContainer] = ()
    detachedSubSchedules: List[BusinessObject] = ()
    options: SchMgtOptions = None
    newTasks: List[GenericAttributesContainer] = ()
    newDependencies: List[GenericAttributesContainer] = ()
    newAssignments: List[GenericAttributesContainer] = ()
    taskUpdates: List[GenericAttributesContainer] = ()
    dependencyUpdates: List[GenericAttributesContainer] = ()
    assignmentUpdates: List[GenericAttributesContainer] = ()
    deletedTasks: List[ScheduleTask] = ()


@dataclass
class ScheduleModifyResponse(TcBaseObj):
    """
    The response of the schedule modification.
    
    :var newTasks: A list of new tasks.
    :var newProxyTasks: A list of new proxytasks.
    :var newDependencies: A list of new dependencies.
    :var newAssignments: A list of new assignments.
    """
    newTasks: List[ScheduleTask] = ()
    newProxyTasks: List[Fnd0ProxyTask] = ()
    newDependencies: List[TaskDependency] = ()
    newAssignments: List[ResourceAssignment] = ()


@dataclass
class ScheduleModifyResponses(TcBaseObj):
    """
    A collection of responses.
    
    :var serviceData: The service data
    :var responses: A collection of reponses.
    """
    serviceData: ServiceData = None
    responses: List[ScheduleModifyResponse] = ()



from tcsoa.gen.Internal_XlsBom._2020_12.services import ImportService as imp0
from tcsoa.base import TcService


class ImportService(TcService):
    getStructureInfoFromExcel = imp0.getStructureInfoFromExcel
    importExcelAndUpdateMappingGrp = imp0.importExcelAndUpdateMappingGrp
    importExcelAndUpdateMappingGrpAsync = imp0.importExcelAndUpdateMappingGrpAsync



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.gen.Internal.Cae._2013_05.StructureManagement import GetCAEPropertyComparisonDetailsResponse
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def getCAEPropertyComparisonDetails(cls, targetBomline: BOMLine) -> GetCAEPropertyComparisonDetailsResponse:
        """
        The attribute comparison in CAE Manager is performed by applying data mapping on the source BOMLine and
        comparing the attribute value resulting from data mapping with the attribute value of the source BOMLine using
        the Accountability Check framework.
        
        This operation retrieves details of the comparison for the provided BOMLine object after the user has executed
        the Accountability Check for attribute comparison in CAE Manager. These results can be displayed in the CAE
        Accountability Check results view or as an Excel report. 
        """
        return cls.execute_soa_method(
            method_name='getCAEPropertyComparisonDetails',
            library='Internal-Cae',
            service_date='2013_05',
            service_name='StructureManagement',
            params={'targetBomline': targetBomline},
            response_cls=GetCAEPropertyComparisonDetailsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportManufaturingFeaturesInput(TcBaseObj):
    """
    input for import manufaturing features
    
    :var context: A BOMLine in the product structure. The connected parts of the imported manufatruing features are
    searched under this line
    :var content: A list of detailed input
    """
    context: BusinessObject = None
    content: List[ImportManufaturingFeaturesScope] = ()


@dataclass
class ImportManufaturingFeaturesScope(TcBaseObj):
    """
    Detailed input for importing manufacturing features
    
    :var xmlFileTicket: The FMS file ticket for the input XML file to be imported
    :var scope: The BOMLine under which the manufaturing features are being searched. Based on the result of the
    search, the import decides whether to create a new manufacturing feature or to update an existing one
    :var container: The BOMLine under which the manufaturing features are going to be imported
    :var importMode: Indicates whether the existing manufacturing features under the container shall be deleted or not.
    The possible values of the import mode are:  keepExistingFeatures The existing discrete manufacturing features
    under the container should not be deleted. refreshWholeContainer   The existing discrete manufacturing features
    under the container may be deleted.
    :var nameRefFileTickets: A list of FMS tickets for the dataset named reference files
    """
    xmlFileTicket: str = ''
    scope: BusinessObject = None
    container: BusinessObject = None
    importMode: str = ''
    nameRefFileTickets: List[str] = ()


@dataclass
class MfgExportToBriefcaseResponse(TcBaseObj):
    """
    This operation returns export log and datasets associated with exported objects.
    
    :var briefcaseFileFMSTicket: FMS ticket of the briefcase file, which can be used to download the briefcase file
    from server to client.
    :var fileTickets: To represent a file ticket and its original file name.
    :var messageIds: List of message IDs.
    :var briefcaseDataSet: A business object of Dataset which includes the out briefcase file in its namedReference.
    After export, a new Dataset will be created. The exported briefcase file will be added to the new Dataset.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors in case
    of failure conditions.
    """
    briefcaseFileFMSTicket: str = ''
    fileTickets: List[str] = ()
    messageIds: List[str] = ()
    briefcaseDataSet: BusinessObject = None
    serviceData: ServiceData = None


"""
A map of strings. Contains names and values.
"""
NamesAndValuesMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Classification._2015_10.Classification import GetClassDefinitionsResponse
from typing import List
from tcsoa.base import TcService


class ClassificationService(TcService):

    @classmethod
    def getClassDefinitions(cls, classIDs: List[str]) -> GetClassDefinitionsResponse:
        """
        This operation returns Classification class definition, its attribute definitions including the details of
        associated Classification KeyLOV (Stxt) objects.
        
        Use cases:
        User wants to get details of classification class along with all associated class attributes. This operation is
        combination of getClassDescriptions and getAttributesForClasses2, but provides information in a different
        format to cater additional information like extended properties, attribute dependency and KeyLOVs (Stxt) 
        associated to class attributes.
        """
        return cls.execute_soa_method(
            method_name='getClassDefinitions',
            library='Classification',
            service_date='2015_10',
            service_name='Classification',
            params={'classIDs': classIDs},
            response_cls=GetClassDefinitionsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2018_05.FullTextSearch import Awp0ModifyFTSSavedSearchInput, Awp0CreateFTSSavedSearchInput
from tcsoa.gen.Internal.AWS2._2015_10.FullTextSearch import Awp0FullTextSavedSearchResponse
from tcsoa.gen.Internal.AWS2._2016_03.Finder import ColumnConfigInput, SaveColumnConfigData
from tcsoa.gen.Internal.AWS2._2018_05.FileMgmt import LoadPlmdTicketReplaceResponse, DatashareManagerReplaceInfo
from tcsoa.gen.Internal.AWS2._2018_05.Finder import FacetSearchInput, SearchResponse5, SearchInput2, FacetSearchResponse
from tcsoa.gen.Internal.AWS2._2018_05.DataManagement import Tile, UnpinObjectsResponse, SaveViewModelEditAndSubmitResponse2, GetCurrentUserGateway2Response, UpdateTilesResponse, PinObjectInput
from tcsoa.gen.Internal.AWS2._2018_05.TCXML import GetDiagnosticInfoForAcctTablesResp
from tcsoa.gen.BusinessObjects import Item, WorkspaceObject
from typing import List
from tcsoa.gen.Internal.AWS2._2017_06.DataManagement import SaveViewModelEditAndSubmitInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from tcsoa.gen.Internal.AWS2._2018_05.Workflow import WorkflowTaskViewModelInput, WorkflowTaskViewModelResponse


class GlobalAlternateService(TcService):

    @classmethod
    def addAlternates(cls, element: WorkspaceObject, alternatesToBeAdded: List[WorkspaceObject]) -> ServiceData:
        """
        Adds objects as alternates to the selected object. Alternates components are parts that are interchangeable
        globally.
        
        Use cases:
        In order to add one or more alternates to the selected object, the user can call addAlternates operation and
        pass in the selected object and a list of business objects to be defined as alternates of the selected object.
        """
        return cls.execute_soa_method(
            method_name='addAlternates',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='GlobalAlternate',
            params={'element': element, 'alternatesToBeAdded': alternatesToBeAdded},
            response_cls=ServiceData,
        )

    @classmethod
    def removeAlternates(cls, element: WorkspaceObject, alternatesToBeRemoved: List[Item]) -> ServiceData:
        """
        Removes alternates from the selected object.
        
        Use cases:
        In order to remove one or more alternates from the selected object, the user can call removeAlternates
        operation and pass in the selected object and a list of alternates to be removed.
        """
        return cls.execute_soa_method(
            method_name='removeAlternates',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='GlobalAlternate',
            params={'element': element, 'alternatesToBeRemoved': alternatesToBeRemoved},
            response_cls=ServiceData,
        )


class WorkflowService(TcService):

    @classmethod
    def getWorkflowTaskViewModel(cls, input: List[WorkflowTaskViewModelInput]) -> WorkflowTaskViewModelResponse:
        """
        Operation to get the properties on EPMTask and Signoff objects in JSON format.  It can also be used to traverse
        the reference properties on EPMTask and Signoff and get properties on the referenced objects.  The output JSON
        is designed to be consumed by the active workspace client panels.  The information needed to be displayed in
        the panels can be obtained by properly constructing the input structure.
        
        Use cases:
        In Active Workspace, Select-Signoff-Task perform panel needs to show the following data:
        - Required Profiles with the count of unstaffed profiles
        - Profile signoffs
        - Adhoc signoffs
        - Quorum data
        
        
        This SOA is designed to get all the above data .  It can also return a logical flag(s) to indicate if the task
        can be completed or not based on which complete button can be displayed in the dialog.This will avoid
        unnecessory complex logic on the client to segregate data( like profiles, profile signoffs and adhoc signoff)
        to display in the panel and to detemine current quorum based on exisiting signoffs etc.
        """
        return cls.execute_soa_method(
            method_name='getWorkflowTaskViewModel',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='Workflow',
            params={'input': input},
            response_cls=WorkflowTaskViewModelResponse,
        )


class TCXMLService(TcService):

    @classmethod
    def installDBTriggersForDataSync(cls, triggerType: int) -> ServiceData:
        """
        Installs Database triggers on POM_object table for tracking newly added objects and (or) deleted objects that
        are used for data synchronization purpose.
        
        Use cases:
        Data indexing uses TCXML as the payload for extracting data from Database. In order to support data
        synchronization, data indexing needs to install Database triggers to track newly added objects and deleted
        objects as the first step of indexing.
        """
        return cls.execute_soa_method(
            method_name='installDBTriggersForDataSync',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='TCXML',
            params={'triggerType': triggerType},
            response_cls=ServiceData,
        )

    @classmethod
    def removeDBTriggersForDataSync(cls, triggerType: int) -> ServiceData:
        """
        Removes the Database triggers on POM_object table for tracking newly added objects and (or) deleted objects
        that are used for data synchronization purpose.
        
        Use cases:
        Data indexing uses TCXML as the payload for extracting data from Database. In order to support data
        synchronization, data indexing needs to install Database triggers to track newly added objects and deleted
        objects as the first step of indexing. This operatoin is a mirror operation to remove the Database triggers
        that were installed by installDBTriggersForDataSync operation.
        """
        return cls.execute_soa_method(
            method_name='removeDBTriggersForDataSync',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='TCXML',
            params={'triggerType': triggerType},
            response_cls=ServiceData,
        )

    @classmethod
    def getDiagnosticInfoForAcctTables(cls, infoType: int) -> GetDiagnosticInfoForAcctTablesResp:
        """
        Returns the accountability tables' and the related POM tables' diagnostic information. The accountability
        tables are used for recording exported data and statuses, as well as data synchronization status and candidates.
        
        Use cases:
        Data indexing uses the operation to retrieve the accountability tables' diagnostic information and reports the
        information to end user. As an environment health check, this helps to understand the underlying tables status
        and diagnose any failure or performance problems.
        """
        return cls.execute_soa_method(
            method_name='getDiagnosticInfoForAcctTables',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='TCXML',
            params={'infoType': infoType},
            response_cls=GetDiagnosticInfoForAcctTablesResp,
        )


class FileMgmtService(TcService):

    @classmethod
    def loadPlmdTicketForReplace(cls, infos: List[DatashareManagerReplaceInfo]) -> LoadPlmdTicketReplaceResponse:
        """
        This operation assists in an asynchronous file replace using the Datashare Manager. When a single file is being
        replaced, the PLMD (Product Lifecycle Management Data) file is downloaded to the Transient volume and the file
        ticket returned. In the case of multiple files being replaced, the output structure is populated with
        additional informaton needed for the Replace operation.
        """
        return cls.execute_soa_method(
            method_name='loadPlmdTicketForReplace',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='FileMgmt',
            params={'infos': infos},
            response_cls=LoadPlmdTicketReplaceResponse,
        )


class FullTextSearchService(TcService):

    @classmethod
    def modifyFullTextSavedSearch(cls, inputs: List[Awp0ModifyFTSSavedSearchInput]) -> Awp0FullTextSavedSearchResponse:
        """
        This operation modifies existing Awp0FullTextSavedSearch objects.
        """
        return cls.execute_soa_method(
            method_name='modifyFullTextSavedSearch',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='FullTextSearch',
            params={'inputs': inputs},
            response_cls=Awp0FullTextSavedSearchResponse,
        )

    @classmethod
    def createFullTextSavedSearch(cls, inputs: List[Awp0CreateFTSSavedSearchInput]) -> Awp0FullTextSavedSearchResponse:
        """
        This operation creates Awp0FullTextSavedSearch objects. Awp0FullTextSavedSearch objects are used to store
        information about a saved search such as search name, search string, search filters, chart input parameters etc.
        """
        return cls.execute_soa_method(
            method_name='createFullTextSavedSearch',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='FullTextSearch',
            params={'inputs': inputs},
            response_cls=Awp0FullTextSavedSearchResponse,
        )


class FinderService(TcService):

    @classmethod
    def performFacetSearch(cls, facetSearchInput: FacetSearchInput) -> FacetSearchResponse:
        """
        This operation fetches facet values in a paged fashion for a selected filter category. It also supports search
        on facet values to get matching facet values for a given text input.
        
        Use cases:
        Use Case 1: User clicks on unpopulated filter category
        Only 200 classification filters are populated and available at the client. To get facet values for an
        unpopulated filter, a search request will be made by passing in the needed criteria. It queries for the facet
        values and returns to the client. The number of values to be returned is controlled by the input parameter
        maxToReturn.
        
        Use Case 2: User performs search for filter values by entering text in filter category search input box. 
        If user types in a value in the filter box, a search request will be made by passing in the needed criteria.
        The input text will be passed into &lsquo;stringValue&rsquo; parameter of the input searchFilterMap. It queries
        for the data and finds the facet values matching the input text. These facet values are included in the
        searchFilterMap of the response. 
        
        Use Case 3: User clicks on "More" to get next set of facet values
        If &lsquo;hasMoreFacetValues&rsquo; flag of the FacetSearchResponse is true, the client will make a search
        request for facet values. Facet values will be queried and returned based on the input criteria.
        """
        return cls.execute_soa_method(
            method_name='performFacetSearch',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='Finder',
            params={'facetSearchInput': facetSearchInput},
            response_cls=FacetSearchResponse,
        )

    @classmethod
    def performSearchViewModel3(cls, searchInput: SearchInput2, columnConfigInput: ColumnConfigInput, saveColumnConfigData: SaveColumnConfigData, inflateProperties: bool, noServiceData: bool) -> SearchResponse5:
        """
        This operation routes a search request to a specific provider specified as providerName in the searchInput
        structures.
        
        The framework allows a custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering. The provider can be a User Inbox retriever or Full Text searcher, for example. The new
        data provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The
        implementation is done using its fnd0performSearch operation.
        
        RuntimeBusinessObject 
        ---- Fnd0BaseProvider (foundation template) 
        -------- Fnd0GetChildrenProvider(foundation template) 
        -------- Awp0FullTextSearchProvider (aws template)
        -------- Awp0TaskSearchProvider (aws template) 
        -------- Aos0TestProvider (aosinternal template) 
        -------- etc.
        
        This operation provides a common interface to send the request to and receive the response from a new data
        provider. Ultimately it allows common framework in UI to support filter, pagination, and sorting. 
        
        This operation allows the caller to send the search input, filter values, and sorting data. These input values
        are then to be passed to the fnd0performSearch operation on the server to collect, sort, and filter its
        results. 
        
        The first two input parameters are important. The first input is the provider name. This is a string to
        represent the type name of RuntimeBusinessObject which this request should be routed to. If the template that
        contains the class is not installed, a partial error 217016 is returned. The second input is the search input
        map. The key is different per each provider. For example, for Full Text searcher, the input key would be a
        searchString. For User Inbox retriever, the input key would be an Inbox Type. The fnd0performSearch
        implementation for each provider takes into account the key name as it is used to store the values in
        OperationInput object. 
        
        The internal property name is passed as part of search input to allow grouping of business objects. The
        grouping allows for added functionality such as color coding, etc. If no internal property name is passed,
        default value is assumed as "object_type" to group the business objects. 
        
        If new list of column config data is passed in to be saved, it will get saved in the database for the login
        user scope. This will be done prior to performing the search. 
        
        The column configuration input has the criteria needed to query for the columns configured for the provided
        client and client scope URI. The final list of columns are identified based on the types returned back in the
        search results and the operation type (Union / Intersection / Configured ) configured for the client scope URI.
        Finally the determined column attributes and the attributes to be inflated passed in as part of search input
        criteria are combined. This combined list of attributes along with the business objects found during search are
        added to the Service data.
        
        Use cases:
        Use Case 1: User clicks on Inbox tile
        A search request will be made by passing in the needed criteria. In this case the provider name is
        "Awp0InboxProvider". It queries for the data and finds the effective columns for the client scope URI passed in
        through column config input. The results are finally shown in the view type selected in client. If table view
        is selected the returned effective columns will be shown.
        
        Use Case 2: User performs search by entering a keyword in global search input box. 
        A search request will be made by passing in the needed criteria that includes the keyword user has typed in. In
        this case the provider name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective
        columns for the client scope URI passed in through column config input. The results are finally shown in the
        view type selected in client. If table view is selected the returned effective columns will be shown. 
        
        Use Case 3: User tries to save new column configurations after performing search. 
        The new list of column configurations will be saved for the login user scope. Later the search request will be
        made by passing in the needed criteria that includes the keyword user had typed in. In this case the provider
        name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective columns for the client
        scope URI passed in through column config input. The results are finally shown in the view type selected in
        client. If table view is selected the returned effective columns will be shown.
        
        ---
        """
        return cls.execute_soa_method(
            method_name='performSearchViewModel3',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='Finder',
            params={'searchInput': searchInput, 'columnConfigInput': columnConfigInput, 'saveColumnConfigData': saveColumnConfigData, 'inflateProperties': inflateProperties, 'noServiceData': noServiceData},
            response_cls=SearchResponse5,
        )


class DataManagementService(TcService):

    @classmethod
    def pinObjects(cls, input: List[PinObjectInput]) -> ServiceData:
        """
        This operation pins the objects represented by the input objects to the user&rsquo;s Gateway.
        
        Use cases:
        Pin an Object while not in the Gateway
        1. User logs into Active Workspace.
        2. User selects a previouslycreated BusinessObject object.
        3. User chooses the "pin" command.
        4. ActiveWorkspace invokes the pinObject service operation, providing the UID of the BusinessObject object to
        pin, along with any optional template ID or parameter information in the list of PinObjectInput structures.
        """
        return cls.execute_soa_method(
            method_name='pinObjects',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def saveViewModelEditAndSubmitWorkflow2(cls, inputs: List[SaveViewModelEditAndSubmitInfo]) -> SaveViewModelEditAndSubmitResponse2:
        """
        This operation saves the modified view model properties for the given input objects and submits the objects to
        a workflow. The workflow is submitted only if all of the save operations are successful. If the save fails for
        a single object  none of the input objects will be submitted to a workflow. This operation saves the
        ViewModelObject and supports dynamic compound properties.
        
        Use cases:
        User can modify the object(s) properties and submit the object(s) to workflow in one operation. This operation
        first saves the modified properties and then initiates the workflow process for all input objects.
        """
        return cls.execute_soa_method(
            method_name='saveViewModelEditAndSubmitWorkflow2',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SaveViewModelEditAndSubmitResponse2,
        )

    @classmethod
    def unpinObjects(cls, returnGateway: bool, uidsToUnpin: List[str]) -> UnpinObjectsResponse:
        """
        This operation unpins the objects represented by the input list of UIDs from the user&rsquo;s Gateway.
        
        Use cases:
        This operation is suited to update the Tile objects on the Active Workspace client gateway page.
        
        Unpin an Object while not in the Gateway
        1. User logs into Active Workspace.
        2. User selects a previously pinned BusinessObject object and chooses the "unpin" command.
        3. Active Workspace invokes unpinObjects operation.
        4. Using the unpinObjects&rsquo; response, Active Workspace displays the updated group of Tile objects that are
        configured for the logged in user.
        
        Unpin a user specific Awp0TileObject
        1. User logs into Active Workspace.
        2. Active Workspace invokes getCurrentUserGateway2 operation.
        3. Displays the groups of Tile objects that are configured for the logged in user.
        4. User selects a Tile object and chooses the "unpin" command.
        5. Active Workspace invokes unpinObjects operation.
        6. Using the unpinObjects&rsquo; response, Active Workspace displays the updated group of Tile objects that are
        configured for the logged in user.
        """
        return cls.execute_soa_method(
            method_name='unpinObjects',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='DataManagement',
            params={'returnGateway': returnGateway, 'uidsToUnpin': uidsToUnpin},
            response_cls=UnpinObjectsResponse,
        )

    @classmethod
    def updateTiles(cls, tiles: List[Tile], groupNames: List[str]) -> UpdateTilesResponse:
        """
        This operation updates the input list of Tile objects from the user&rsquo;s Gateway.  This includes moving Tile
        objects from one group to another, as well as updating properties on Tile objects such as the size and ordering
        of Tile objects.
        
        Use cases:
        This operation is suited to update the Tile objects on the Active Workspace client gateway page.
        
        Update the user specific Awp0Tile objects.
        1. User logs into Active Workspace.
        2. Active Workspace invokes getCurrentUserGateway2 operation.
        3. Displays the groups of Tile objects that are configured for the logged in user.
        4. User selects a Tile object and updates the Tile&rsquo;s size, order, and group.
        5. Active Workspace invokes updateTiles operation.
        6. Using the updateTiles&rsquo; response, Active Workspace displays the updated group of Tile objects that are
        configured for the logged in user.
        """
        return cls.execute_soa_method(
            method_name='updateTiles',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='DataManagement',
            params={'tiles': tiles, 'groupNames': groupNames},
            response_cls=UpdateTilesResponse,
        )

    @classmethod
    def getCurrentUserGateway2(cls) -> GetCurrentUserGateway2Response:
        """
        The operation looks for the Awp0TileCollection with current logged in user as scope and returns information
        about the Awp0Tile objects associated to it.
        
        Use cases:
        This operation is suited to display the Awp0Tile objects on the Active Workspace client gateway page. The Tile
        objects that are returned by this operation are the ones which are configured for the logged in user based on
        current group, role, and project.
        
        Display the user specific Awp0Tile objects during startup
        1. User logs into Active Workspace.
        2. Active Workspace invokes getCurrentUserGateway2 operation.
        3. Displays the groups of Tile objects that are configured for the logged in user.
        
        Display the user specific Awp0Tile objects during context change
        1.  User changes the group/role/project in Active Workspace.
        2.  Active Workspace invokes getCurrentUserGateway2 operation.
        3.  Displays the groups of Tile objects that are configured for the logged in user based on current group,
        role, and project.
        """
        return cls.execute_soa_method(
            method_name='getCurrentUserGateway2',
            library='Internal-AWS2',
            service_date='2018_05',
            service_name='DataManagement',
            params={},
            response_cls=GetCurrentUserGateway2Response,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ReleaseStatus, Effectivity
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReleaseStatusEffectivityInput(TcBaseObj):
    """
    This structure contains effectivities to be attached to the  release status.
    
    :var status: The ReleaseStatus for which Effectivity objects are to be added or removed.
    :var effectivities: A list of Effectivity objects which are added or removed  to/from status.
    :var addOrRemove: Indicates whether to add or remove Effectivity(s). 
    When true, Effectivity is added.Otherwise,  Effectivity is removed.
    """
    status: ReleaseStatus = None
    effectivities: List[Effectivity] = ()
    addOrRemove: bool = False



from __future__ import annotations

from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GroupMembershipInput2(TcBaseObj):
    """
    A structure containing input to retrieve group membership information.
    
    :var userId: The string specifies the user_id to search for within the organization. An example is: User wants to
    search for GroupMember objects which have a User with user_id equal to this input. Partial string supported. This
    is an optional input.
    :var userName: The string specifies the user_name to search for within the organization. An example is: User wants
    to search for GroupMember objects which have a User with user_name equal to this input. Partial string supported.
    This is an optional input.
    :var groupName: The string specifies the Group name to search within the organization. An example is: User wants to
    search for GroupMember objects which have a Group with name equal to this input. Partial string supported. This is
    an optional input.
    :var roleName: The string specifies the Role name to search within the organization. An example is: User wants to
    search for GroupMember objects which have a Role with role_name equal to this input. Partial string supported. This
    is an optional input.
    :var searchForSubGroup: A flag input to specify whether to search for sub-group of given group in groupName input.
    If the groupName is empty or contains  wild card( "*" ), then this input will be ignored. This is an optional input.
    :var startIndex: This input is used for pagination. User can specify the the start index of the next chunk of
    objects to return. An example of this input is: This operation finds 100 GroupMember objects. User want to get
    first 50 searched objects. So here the startIndex would be 0. For next chunk of searched objects this input would
    be 51.
    Note: Value for this input should be a positive integer. If the value is negative number then it is considered as 0.
    
    :var maxToReturn: This input is used for pagination. This input specifies the maximum number of found objects to
    return.
    Note: Value for this input should be a positive integer. If the value is 0 or negative number then this input will
    be ignored and all the found objects will be returned.
    
    :var maxToLoad: This input is used for pagination. This input specifies the maximum number of found objects to load.
    Note: Value for this input should be a positive integer. If the value is 0 or negative number then this input will
    be ignored and all the found objects will be loaded.
    """
    userId: str = ''
    userName: str = ''
    groupName: str = ''
    roleName: str = ''
    searchForSubGroup: bool = False
    startIndex: int = 0
    maxToReturn: int = 0
    maxToLoad: int = 0



from __future__ import annotations

from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class DeleteUsersInput(TcBaseObj):
    """
    Input structure to delete User object.
    
    :var userId: The User ID of the User to be deleted.
    :var newOwningUser: The User ID to assign ownership of the objects currently owned by 'userId'. This value is
    ignored if 'deleteObjects' is true. A value must be provided if 'deleteObjects' is false.
    :var newOwningGroup: The name of the Group to assign Group ownership of object currently owned by the 'userId'. The
    'newOwningUser' must be a member of this group.The Group name is ignored if 'deleteObjects' is true. If
    'deleteObjects' is false and value is not given then the default group of 'newOwningUser' is used.
    :var deleteObjects: If true, the objects owned by 'userId' are deleted. If false, the objects are kept and their
    ownership is transferred to 'newOwningUser' and 'newOwningGroup'.
    """
    userId: str = ''
    newOwningUser: str = ''
    newOwningGroup: str = ''
    deleteObjects: bool = False



from __future__ import annotations

from tcsoa.gen.Ai._2014_12.Ai import GetMappedAppRefsResponse, GetMappedAppRefsInput, CreateAppInterfaceRecordInput, CreateAppInterfaceRecordsResponse
from typing import List
from tcsoa.base import TcService


class AiService(TcService):

    @classmethod
    def createApplicationInterfaceRecords(cls, input: List[CreateAppInterfaceRecordInput]) -> CreateAppInterfaceRecordsResponse:
        """
        This operation creates RecordObjects for the specified labels in the MasterRecord associated with the input
        AppInterface object. Input labels are PLMXML style label strings of ApplicationRef element related to
        Application type "Teamcenter". 
        
        Use cases:
        Use Case 1: Creating RecordObjects for specified PLMXML labels.
        This operation should be used to create RecordObjects for  Teamcenter Application References which were not
        exported via PLMXML. Typical case would be Light Weight BOM APIs are used to get the data, but, later there is
        a need to do a PLMXML import using AppInterface object.
        
        """
        return cls.execute_soa_method(
            method_name='createApplicationInterfaceRecords',
            library='Ai',
            service_date='2014_12',
            service_name='Ai',
            params={'input': input},
            response_cls=CreateAppInterfaceRecordsResponse,
        )

    @classmethod
    def getMappedApplicationRefs(cls, appRefs: List[GetMappedAppRefsInput]) -> GetMappedAppRefsResponse:
        """
        This operation searches for objects with specified Application References and returns the matching Application
        References with specified Application names. Application Reference is a 3-tuple construct with name, label and
        version strings. This is used in PLMXML exchange between Teamcenter and target applications to uniquely
        identity Teamcenter entities like Item, ItemRevision, Form objects etc.
        
        Use cases:
        Use Case 1: Getting Teamcenter Application References for non Teamcenter Application References.
        This operation can be used to fetch the Application References of Teamcenter given non Teamcenter Application
        References. These are typically used in PLMXML interchange.
        Use Case 2: Getting non Teamcenter Application References for Teamcenter Application References.
        This operation can be used to fetch the Application References of non Teamcenter  Application References given
        Teamcenter Application References. These are typically used in PLMXML interchange.
        """
        return cls.execute_soa_method(
            method_name='getMappedApplicationRefs',
            library='Ai',
            service_date='2014_12',
            service_name='Ai',
            params={'appRefs': appRefs},
            response_cls=GetMappedAppRefsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Administration._2012_10.IRM import GetSessionValuesResponse, AMImpactedObjectsResponse
from tcsoa.base import TcService


class IRMService(TcService):

    @classmethod
    def getSessionValues(cls) -> GetSessionValuesResponse:
        """
        This operation gets all the session information in the form of key values map.  Each key corresponds to
        particular session attribute like user, roles, groups, project teams, and licenses.  For each entry in the keys
        array, the corresponding entry in the values array contains the values for that specific session attribute.
        Session information returned from this SOA operation is used during read expression evaluation in external
        clients to determine the READ privilege to the current logged in user on the indexed Teamcenter data.
        """
        return cls.execute_soa_method(
            method_name='getSessionValues',
            library='Administration',
            service_date='2012_10',
            service_name='IRM',
            params={},
            response_cls=GetSessionValuesResponse,
        )

    @classmethod
    def getAMImpactedObjects(cls, filterByIndexedStatus: bool) -> AMImpactedObjectsResponse:
        """
        This operation lists the business objects whose read access is impacted by the changes in Access Manager (AM)
        rule tree. The rule tree changes considered are limited to those made after the previous call to this operation.
        This operation is usually called periodically and the objects whose read access privilege is modified due to
        the changes to AM rule tree between the previous call and the current one are determined and returned.
        Optionally, this operation returns the set of objects which is the intersection of objects impacted by AM rule
        changes and objects previously indexed. Previously indexed objects are stored in ACCT_TABLE table.
        AM rule configuration changes need re-login to refresh the in memory rule registry cache. Hence, until re-login
        after the AM rule changes, this operation will return ZERO objects.
        """
        return cls.execute_soa_method(
            method_name='getAMImpactedObjects',
            library='Administration',
            service_date='2012_10',
            service_name='IRM',
            params={'filterByIndexedStatus': filterByIndexedStatus},
            response_cls=AMImpactedObjectsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Group, ClosureRule, ImanFile, PropertySet, Filter, TransferOptionSet, POM_imc, TransferMode, User, Role, PIEActionRule
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetActionRulesResponse(TcBaseObj):
    """
    The 'GetActionRulesResponse' structure defines the response from 'getActionRules' operation. It contains vector of
    action rule object references that were created on the server that satisfy the input criteria scope and schema
    format.
    
    :var actionRuleObjects: A list of action rule objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    actionRuleObjects: List[PIEActionRule] = ()
    serviceData: ServiceData = None


@dataclass
class GetAllTransferOptionSetsResponse(TcBaseObj):
    """
    The 'GetAllTransferOptionSetsResponse' structure defines the response from 'getAllTransferOptionSets' operation. It
    contains vector of transfer option set object references that were present on the server.
    
    :var optionSetObjects: A list of transfer option sets.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    optionSetObjects: List[TransferOptionSet] = ()
    serviceData: ServiceData = None


@dataclass
class GetAvailableTransferOptionSetsInputData(TcBaseObj):
    """
    The structure 'GetAvailableTransferOptionSetsInputData' defines input criteria for the operation
    'getAvailableTransferOptionSets'. The values of 'isPush', 'isExport' are to be known before calling this operation.
    
    Case 1: Remote Export    - on-line  --- 'isPush' = True   'isExport' = True
    Case 2: Remote Import    - on-line  --- 'isPush' = False  'isExport' = True
    Case 3: Briefcase Export - off-line --- 'isPush' = True   'isExport' = True
    case 4: Briefcase Import - off-line --- isPush = NA     isExport = False
    
    For 2007.1, the site can not be more than one for the Push case. The access rules will be evaluated thru RBF (Rules
    Based Framework) for the given user, group, role and all the option sets that satisfy the criteria will be returned
    
    :var user: The user reference that executing this API.
    :var group: The group reference that the executing user belongs to.
    :var role: The role reference that the executing user belongs to.
    :var site: The site that the user want to export to.
    :var isPush: Option that control the online or offline.
    :var isExport: Option that control the data transfer direction.
    """
    user: User = None
    group: Group = None
    role: Role = None
    site: List[POM_imc] = ()
    isPush: bool = False
    isExport: bool = False


@dataclass
class GetAvailableTransferOptionSetsResponse(TcBaseObj):
    """
    The 'GetAvailableTransferOptionSetsResponse' structure defines the response from 'getAvailableTransferOptionSets'
    operation. It contains vector of transfer option set object references that satisfies the input criteria.
    
    :var optionSetObjects: A list of transfer option set objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    optionSetObjects: List[TransferOptionSet] = ()
    serviceData: ServiceData = None


@dataclass
class GetClosureRulesResponse(TcBaseObj):
    """
    The 'GetClosureRulesResponse' structure defines the response from 'getClosureRules' operation. It contains vector
    of closure rule object references that were created on the server that satisfy the input criteria scope and schema
    format.
    
    :var closureRuleObjects: A list of closure rule objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    closureRuleObjects: List[ClosureRule] = ()
    serviceData: ServiceData = None


@dataclass
class GetFilterRulesResponse(TcBaseObj):
    """
    The 'GetFilterRulesResponse' structure defines the response from' getFilterRules' operation. It contains vector of
    filter rule object references that were created on the server that satisfy the input criteria scope and schema
    format.
    
    :var filterRuleObjects: A list of filter rule objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    filterRuleObjects: List[Filter] = ()
    serviceData: ServiceData = None


@dataclass
class GetPLMXMLRuleInputData(TcBaseObj):
    """
    The structure 'GetPLMXMLRuleInputData' is used for many operations related to PLMXML Admin application. This
    structure specifies input criteria for get list methods for Transfermode, closure rule, filter rule action rule.
    The get list operation is expected to return all rule objects based on the given scope and schema format.
    
    :var scope: The direction that scope rule is used. The allowed values are"EXPORT", "IMPORT" or "ALL". When the
    value is "EXPORT", only export based scope rules will be queried. Value "ALL" implies both export and import scope
    rules will be queried.
    :var schemaFormat: The schema format associated with the scope rule. The allowed values are "PLMXML" or  "TCXML" or
    "ALL". When the value is "PLMXML", only PLMXML based scope rules will be queried. Value "ALL" implies both PLMXML
    and TCXML scope rules will be queried.
    """
    scope: str = ''
    schemaFormat: str = ''


@dataclass
class GetPropertySetsResponse(TcBaseObj):
    """
    The 'GetPropertySetsResponse' structure defines the response from' getPropertySets' operation. It contains vector
    of property set object references that were created on the server that satisfy the input criteria scope. (Schema
    format is not applicable in input)
    
    :var propertySetObjects: A list of property set objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    propertySetObjects: List[PropertySet] = ()
    serviceData: ServiceData = None


@dataclass
class GetTransferModesResponse(TcBaseObj):
    """
    The 'GetTransferModesResponse' structure defines the response from 'getTransferModes' operation. It contains vector
    of transfer mode object references that were created on the server that satisfy the input criteria scope and schema
    format.
    
    :var transferModeObjects: A list of transfer mode objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    transferModeObjects: List[TransferMode] = ()
    serviceData: ServiceData = None


@dataclass
class NamesAndValue(TcBaseObj):
    """
    NameAndValue structure represents a generic name-value pair
    
    :var name: The name of the name-value pair
    :var value: The value of the name-value pair
    """
    name: str = ''
    value: str = ''


@dataclass
class OptionInputData(TcBaseObj):
    """
    Transfer Option Set is a the object which holds complete configuration
    information about the data transfer. It is basically collection of options.
    Options are grouped in UI to give better readability for the options
    The options will be grouped in GUI based on the group name. The default
    values will be true or false for 2007.1. If the readOnlyFlag is set on a
    particular option only "dba" can override the value during transfer. it will
    be shown to the regular user but read-only
    
    The OptionInputData structure defines complete data for one option (Symbol)
    
    :var realName: The real variable name that used in code.
    :var displayName: Specify the name that user see in the transfer option setting .
    :var description: The description for the transfer option.
    :var groupName: The group that the tranfer option belongs to in the whole transfer option set.
    :var defaultValue: The default value for this transfer option.
    :var readOnlyFlag: Specify that wheter this transfer option is modified in transfer option setting. If the value is
    true, this option will be unmodifiable in the transfer option setting.
    """
    realName: str = ''
    displayName: str = ''
    description: str = ''
    groupName: str = ''
    defaultValue: str = ''
    readOnlyFlag: bool = False


@dataclass
class RequestImportFromOfflinePackageResponse(TcBaseObj):
    """
    RequestImportFromOfflinePackageResponse structure defines the response from requestImportFromOfflinePackage
    operation. It contains message Id of the request and partial errors and objects that are imported.
    
    :var msgId: Message Id of this request, to be used to check the status import seesion run at Global Service.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors listed
    above in case of failure conditions.
    """
    msgId: str = ''
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateActionRuleInputData(TcBaseObj):
    """
    The 'CreateOrUpdateActionRuleInputData' specifies complete data for creating an action rule.
    
    :var name: The name of the action rule which can be 32 characters long.
    :var description: The description of the action rule which can be 240 characters long.
    :var scope: The action rule is for import or export.  The value can be "IMPORT" or "EXPORT".
    :var schemaFormat: The schema for the imported/exported xml file. The value can be "TCXML" or "PLMXML".
    :var actionLocation: The  location where the action happens. The value could be "PREACTION", "DURINGACTION" or
    "POSTACTION".
    :var actionName: The Function handler which will be invoked when this rule is executed. For how to create a
    function for a action rule, please refer to PLM XML Import Export Administration Guide.
    :var actionRuleToUpdate: Holds the reference of the action rule in case of modification.
    """
    name: str = ''
    description: str = ''
    scope: str = ''
    schemaFormat: str = ''
    actionLocation: str = ''
    actionName: str = ''
    actionRuleToUpdate: PIEActionRule = None


@dataclass
class CreateOrUpdateActionRuleResponse(TcBaseObj):
    """
    The 'CreateOrUpdateActionRuleResponse' structure defines the response from 'createOrUpdateActionRules' operation.
    It contains vector of action rule object references that were created on the server.
    
    :var actionRuleObjects: A list of created or modified action rule objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    actionRuleObjects: List[PIEActionRule] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateClosureRuleInputData(TcBaseObj):
    """
    The 'CreateOrUpdateClosureRuleInputData' specifies complete data for creating a closure rule.
    
    :var name: The name of the closure rule which can be 32 characters long.
    :var description: The description of the closure rule which can be 240 characters long.
    :var scope: The closure rule is for import or export.  The value can be "IMPORT" or "EXPORT".
    :var schemaFormat: The schema for the imported/exported xml file. The value can be "TCXML" or "PLMXML".
    :var comments: The comments for every clause in this closure rule which can be 240 characters long.
    :var depth: The depth for every clause in this closure rule.
    :var clauses: The clause contents of closure rule which can be 240 characters long.
    :var closureRuleToUpdate: Holds the reference of the closure rule in case of modification.
    """
    name: str = ''
    description: str = ''
    scope: str = ''
    schemaFormat: str = ''
    comments: List[str] = ()
    depth: List[int] = ()
    clauses: List[str] = ()
    closureRuleToUpdate: ClosureRule = None


@dataclass
class CreateOrUpdateClosureRuleResponse(TcBaseObj):
    """
    The 'CreateOrUpdateClosureRuleResponse' structure defines the response from 'createOrUpdateClosureRules' operation.
    It contains vector of closure rule object references that were created on the server.
    
    :var closureRuleObjects: A list of created or modified closure rule objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    closureRuleObjects: List[ClosureRule] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateFilterRuleInputData(TcBaseObj):
    """
    The 'CreateOrUpdateFilterRuleInputData' specifies complete data for creating a filter rule.
    
    :var name: The name of the filter rule which can be 32 characters long.
    :var description: The description of the filter rule which can be 240 characers long.
    :var scope: The filter rule is for import or export. The value can be "IMPORT" or "EXPORT".
    :var schemaFormat: The schema for the imported/exported xml file. The value can be "TCXML" or "PLMXML".
    :var clauses: The  clauses contents of filter rule which can be 128 characters long.
    :var filterRuleToUpdate: Holds the reference of the filter rule in case of modification.
    """
    name: str = ''
    description: str = ''
    scope: str = ''
    schemaFormat: str = ''
    clauses: List[str] = ()
    filterRuleToUpdate: Filter = None


@dataclass
class CreateOrUpdateFilterRuleResponse(TcBaseObj):
    """
    The 'CreateOrUpdateFilterRuleResponse' structure defines the response from 'createOrUpdateFilterRules' operation.
    It contains vector of filter rule object references that were created on the server.
    
    :var filterRuleObjects: A list of created or modified filter rule objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    filterRuleObjects: List[Filter] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdatePropertySetInputData(TcBaseObj):
    """
    The 'CreateOrUpdatePropertySetInputData' specifies complete data for creating a property set.
    
    :var name: The name of the property set which can be 32 characters long.
    :var description: The description of the property set which can be 240 characters long.
    :var scope: The property set is for import or export. The value can be "IMPORT" or "EXPORT".
    :var clauses: The clause contents of property set which can be 128 characters long.
    :var propertySetToUpdate: Holds the reference of the property set in case of modification.
    """
    name: str = ''
    description: str = ''
    scope: str = ''
    clauses: List[str] = ()
    propertySetToUpdate: PropertySet = None


@dataclass
class CreateOrUpdatePropertySetRuleResponse(TcBaseObj):
    """
    The 'CreateOrUpdatePropertySetResponse' structure defines the response from createOrUpdatePropertySets operation.
    It contains vector of property sets object references that were created on the server.
    
    :var propertySetObjects: A list of created or modified property set objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    propertySetObjects: List[PropertySet] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateTransferModeInputData(TcBaseObj):
    """
    The 'CreateOrUpdateTransferModeInputData' specifies complete data for creating a transfer mode.
    
    :var name: The name of the transfer mode which can be 128 characters long.
    :var description: The description of the transfer mode which can be 240 characters long.
    :var xsltFiles: The style sheet file reference for the transfer mode.
    :var configObjs: The revision rule reference for the transfer mode.
    :var actionList: The action rule reference for the transfer mode.
    :var tmToUpdate: Holds the reference of the transfer mode in case of modification.
    :var contextString: The context string used in export/import. The value can be 240 characters long.
    :var direction: Defines the transfer mode is for import or export.  The value can be "IMPORT" or "EXPORT".
    :var schemaFormat: The schema for the imported/exported xml file. The value can be "TCXML" or "PLMXML".
    :var isIncremental: Specifies whether transfermode can be used in incremental data transfer or not.
    :var isMultiSite: Specifies whether transfermode is multisite or not.
    :var closurerule: The closure rule reference for the transfer mode.
    :var filter: The filter rule reference for the transfer mode.
    :var propertyset: The property set reference for the transfer mode.
    """
    name: str = ''
    description: str = ''
    xsltFiles: List[ImanFile] = ()
    configObjs: List[BusinessObject] = ()
    actionList: List[BusinessObject] = ()
    tmToUpdate: TransferMode = None
    contextString: str = ''
    direction: str = ''
    schemaFormat: str = ''
    isIncremental: bool = False
    isMultiSite: bool = False
    closurerule: ClosureRule = None
    filter: Filter = None
    propertyset: PropertySet = None


@dataclass
class CreateOrUpdateTransferModeResponse(TcBaseObj):
    """
    The 'CreateOrUpdateTransferModeResponse' structure defines the response from 'createOrUpdateTransferModes'
    operation. It contains vector of transfer mode object references that were created on the server.
    
    :var transferModeObjects: A list of created or modified transfer mode objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    transferModeObjects: List[TransferMode] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateTransferOptionSetInputData(TcBaseObj):
    """
    The CreateOrUpdateTransferOptionSetInputData structure defines the input data for creating a
    TransferOptionSetObject.
    
    :var name: The name of the transfer opton set.
    :var description: The description of the transfer opton set.
    :var publiclyVisible: Specifies whether it is public or private. If the value is true, the transfer option set will
    be visibale and used for all user.
    :var siteId: Specifies the site reference (local site or remote site).
    :var transfermode: Specifies the transfer mode reference.
    :var options: The vector of options (See OptionInputData structure).
    :var optionSetObjectToModify: This holds the reference to transfer option set object in case of modification.
    """
    name: str = ''
    description: str = ''
    publiclyVisible: bool = False
    siteId: POM_imc = None
    transfermode: TransferMode = None
    options: List[OptionInputData] = ()
    optionSetObjectToModify: TransferOptionSet = None


@dataclass
class CreateOrUpdateTransferOptionSetResponse(TcBaseObj):
    """
    The CreateOrUpdateTransferOptionSetResponse structure defines the response from
    createOrUpdateTransferOptionSets operation. It contains vector of TransferOptionSet
    object references that were created on the server.
    
    :var transferOptionSets: A list of created or modified transfer option set objects.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    transferOptionSets: List[TransferOptionSet] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Manufacturing._2017_05.DataManagement import GetFutureRevisionsResponse, GetFutureRevisionsIn
from typing import List
from tcsoa.gen.Internal.Manufacturing._2017_05.ImportExport import NamesAndValuesMapAsync
from tcsoa.gen.Internal.Manufacturing._2017_05.StructureManagement import SyncMasterAndAlternativeInputInfo, SyncMasterAndAlternativeResponse
from tcsoa.base import TcService


class ImportExportService(TcService):

    @classmethod
    def importFromBriefcaseAsync(cls, fmsTicket: str, optionSetTag: BusinessObject, optionNamesAndValues: NamesAndValuesMapAsync, sessionOptionAndValues: NamesAndValuesMapAsync) -> None:
        """
        This operation is applicable specifically for Manufacturing Process Planner MPP application.
        This operation performs following operation
        Teamcenter::Soa::GlobalMultiSite::_2008_06::ImportExport importObjectsFromOfflinePackage 
        This is asynchronous implementation for the Teamcenter::Soa::Manufacturing::_2017_05::ImportExport
        importFromBriefcase service operation.
        
        Use cases:
        Use Case 1: Importing objects from briefcase asynchronously
        This operation can be used in Manufacturing Process Planner (MPP) application to import Briefcase file into
        Teamcenter in the background. For this operation to be invoked, user has to select the option import in
        background present in import dialog.  Briefcase file is a zipped file containing TC XML and data set files. The
        TC XML file specifies the object to be imported. The import dialog presents various option sets to control the
        objects during import.
        """
        return cls.execute_soa_method(
            method_name='importFromBriefcaseAsync',
            library='Internal-Manufacturing',
            service_date='2017_05',
            service_name='ImportExport',
            params={'fmsTicket': fmsTicket, 'optionSetTag': optionSetTag, 'optionNamesAndValues': optionNamesAndValues, 'sessionOptionAndValues': sessionOptionAndValues},
            response_cls=None,
        )

    @classmethod
    def exportToBriefcaseAsync(cls, reason: str, sites: List[BusinessObject], objects: List[BusinessObject], transferOptionSet: BusinessObject, optionNameAndValues: NamesAndValuesMapAsync, sessionOptionNamesAndValues: NamesAndValuesMapAsync) -> None:
        """
        This operation is applicable specifically for Manufacturing Process Planner MPP application.
        This operation performs following operation
        Teamcenter::Soa::GlobalMultiSite::_2008_06::ImportExport exportObjectsToOfflinePackage 
        In addition, it creates internal objects which are helpful in supplier collaboration use cases for
        manufacturing objects. This is asynchronous implementation for the
        Teamcenter::Soa::Manufacturing::_2016_03::ImportExport exportToBriefcase service operation.
        
        Use cases:
        Use Case 1: Exporting objects to the briefcase asynchronously by transferring the ownership to the supplier.
        User wants to export Collaboration Context (CC) object in MPP to the briefcase to be used by the supplier at
        remote site. 
        The CC may contain product structure(s), bill of processes (BOP) such as plant BOP, plant structure etc.
        While exporting, the user wants to transfer ownership of few objects in the CC to the supplier so that the
        supplier can make changes to those objects on the other site.
        A user selects the CC and uses the menu option Tools, Export To, Briefcase...
        Menu option opens a dialog that allows user to set a destination site, a transfer option set, a list of
        traverse options and a list of session options. User has to select export in background option present in
        export dialog for invoking this operation.
        In this case, all the objects which can be traversed by the transfer option set and session options will be
        exported into an output TC XML file. 
        The files and datasets related to exported objects will be downloaded and packed into the briefcase file along
        with the TC XML file.
        
        In addition, the internal object, Appearance Path Node (APN) will be created for the identified BOMLine objects
        in the CC. The BOMLine objects are identified based on the preference MERuleForBriefcaseExport.
        """
        return cls.execute_soa_method(
            method_name='exportToBriefcaseAsync',
            library='Internal-Manufacturing',
            service_date='2017_05',
            service_name='ImportExport',
            params={'reason': reason, 'sites': sites, 'objects': objects, 'transferOptionSet': transferOptionSet, 'optionNameAndValues': optionNameAndValues, 'sessionOptionNamesAndValues': sessionOptionNamesAndValues},
            response_cls=None,
        )


class StructureManagementService(TcService):

    @classmethod
    def syncMasterAndAlternative(cls, syncMasterAndAlternativeInputInfo: SyncMasterAndAlternativeInputInfo) -> SyncMasterAndAlternativeResponse:
        """
        This service operation synchronizes the source and target Collaboration Context (CC). Source and target can be
        either master CC or Alternative (CC with clone structures).Any newly available Study in source CC is
        synchronized with the target CC. During synchronization a new Study is created in target CC. Also, new
        operations in Plant BOP of source CC is synchronized with the Plant BOP of target CC.
        
        Use cases:
        An Alternative is created by cloning Plant Bill Of Process (BOP) and/or Plant structure from the master CC.
        - Use Case 1: User can create a Shared or Isolated Study using a Plant BOP from Alternative, select the
        Alternative in the CC tree view and use the synchronize command from context menu to synchronize the master CC
        with the selected Alternative. As a result a new Study is created in master CC.Based on whether equivalent
        object from source Study is available in Plant BOP of master CC, the object in the newly created Study is
        either copied or cloned from Plant BOP of master CC. If newly created Study is a Isolated Study, then objects
        are always cloned.
        - Use Case 2: User can create new opertaion(s) in Alternative Plant BOP, select the newly added operation(s)
        and use synchronize command to synchronize the master CC Plant BOP. As a result the new operation(s) are
        created in master CC Plant BOP.
        
        """
        return cls.execute_soa_method(
            method_name='syncMasterAndAlternative',
            library='Internal-Manufacturing',
            service_date='2017_05',
            service_name='StructureManagement',
            params={'syncMasterAndAlternativeInputInfo': syncMasterAndAlternativeInputInfo},
            response_cls=SyncMasterAndAlternativeResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def getFutureRevisions(cls, input: List[GetFutureRevisionsIn]) -> GetFutureRevisionsResponse:
        """
        This operation finds future revisions of the revision configured in by the input BOMLine based on the current
        RevisionRule. If the input BOMLine is Mfg0BvrProcess  subtype, it also finds the future revisions of its direct
        children which are of type Mfg0BvrProcess or Mfg0BvrOperation.  For input BOMLine objects not of Mfg0BvrProcess
        type this operation only finds the future revisions for the input line. The future revisions are found based on
        the current RevisionRule date or unit Effectivity. This operation finds future revisions by advancing the
        RevisionRule object&rsquo;s effectivity and looking for new revisions that would be configured in during that
        effectivity interval. The RevisionRule must have either a unit or date Effectivity clause and at least one
        &lsquo;Has Status&rsquo; clause which is configured by effective date or unit.  When using unit effectivity,
        the future revisions&rsquo; end item must match the current RevisionRule end item in order to be considered.
        Currently, multi-unit effectivity is not supported
        
        Use cases:
        Rev Rule: Has Status(Pending, Configured using Unit number).  End item 1.
        Rev A &ndash; Pending 1 &amp;UP, end item 1
        Rev B &ndash; Pending 5-10, end item 1
        Rev C &ndash; Pending 22-23, end item 1
        Revision Rule Unit Effectivity: 5 &ndash; Rev B configured
        
        Current Revision    Future Revisions &hellip;
             5                           11    22    24
             B                           A     C     A
        
        In this use case, this operation will return the following :
        Current Revision B
           Configuring Pending
              Configuring Effectivity 
                 Range:5-10
                 Transition Effectivity:5
        Future Revision A
           Configuring Pending
              Configuring Effectivity 
                 Range:1 &ndash; UP
                 Transition Effectivity:11,24
        Future Revision C
           Configuring Pending
              Configuring Effectivity 
                 Range:22 &ndash; 23
                 Transition Effectivity:22
        """
        return cls.execute_soa_method(
            method_name='getFutureRevisions',
            library='Internal-Manufacturing',
            service_date='2017_05',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GetFutureRevisionsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TC_Project
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ModifyProjectsInfo(TcBaseObj):
    """
    This structure holds the TC_Project object and the information required to modify the project.
    
    :var sourceProject: The TC_Project object  to be modified.
    :var projectInfo: A ProjectInformation structure.
    :var clientId: Unique identifier for the client to track any errors.
    """
    sourceProject: TC_Project = None
    projectInfo: ProjectInformation = None
    clientId: str = ''


@dataclass
class ProjectClientId(TcBaseObj):
    """
    This structure holds TC_Project object and corresponding client id.
    
    :var tcProject: The TC_Project object whose team members need to be retrieved.
    :var clientId: Unique identifier used by the client to track any errors.
    """
    tcProject: TC_Project = None
    clientId: str = ''


@dataclass
class ProjectInformation(TcBaseObj):
    """
    Structure that holds the information required to create the destination project.
    
    :var projectId: The project ID of  the project  to be created.
    :var projectName: The name of the project to be created.
    :var projectDescription: The description of the project to be created.
    :var useProgramContext: The value of useProgramContext attribute on TC_Project.
    :var active: The value of active attribute on TC_Project.
    :var visible: The value of visible attribute on TC_Project.
    :var teamMembers: A list of TeamMemberInfo structures.
    :var clientId: Unique identifier used by the client to track any errors.
    """
    projectId: str = ''
    projectName: str = ''
    projectDescription: str = ''
    useProgramContext: bool = False
    active: bool = False
    visible: bool = False
    teamMembers: List[TeamMemberInfo] = ()
    clientId: str = ''


@dataclass
class ProjectOpsOutput(TcBaseObj):
    """
    Structure that holds the TC_Project object.
    
    :var project: The created TC_Project object.
    :var clientId: Unique identifier used by the client to track any errors.
    """
    project: TC_Project = None
    clientId: str = ''


@dataclass
class ProjectOpsResponse(TcBaseObj):
    """
    Response from the project create, modify operations.
    
    :var projectOpsOutputs: Vector of ProjectOpsOuput objects.
    :var serviceData: Service data with the partial error information
    """
    projectOpsOutputs: List[ProjectOpsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ProjectTeamData(TcBaseObj):
    """
    This structure holds team member information for a single project.
    
    :var project: The TC_Project object for which team members are obtained.
    :var regularMembers: A list of non privileged members of the given project.
    :var projectTeamAdmins: A  list of project team adminstrators of the given project.
    :var privMembers: A list of privileged members of the given project.
    """
    project: TC_Project = None
    regularMembers: List[BusinessObject] = ()
    projectTeamAdmins: List[BusinessObject] = ()
    privMembers: List[BusinessObject] = ()


@dataclass
class ProjectTeamsResponse(TcBaseObj):
    """
    This structure holds team member information for all the given projects.
    
    :var projectTeams: List of ProjectTeamData objects one for each of the given projects.
    :var serviceData: A standard ServiceData object.
    """
    projectTeams: List[ProjectTeamData] = ()
    serviceData: ServiceData = None


@dataclass
class TeamMemberInfo(TcBaseObj):
    """
    A structure containing team member information.
    
    :var teamMember: The team member of a project.
    :var teamMemberType: A value indicating the teamMember type. Valid values are 0, 1 and 2.
    0 = Team member (teamMember is a GroupMember or Group object)
    1 = Privileged user (teamMember is a User object)
    2 = Team administrator (teamMember is a User object)
    """
    teamMember: BusinessObject = None
    teamMemberType: int = 0


@dataclass
class CopyProjectsInfo(TcBaseObj):
    """
    Structure that holds project information required to create a new TC_Project object using this operation.
    
    :var sourceProject: The TC_Project of  a project  to be copied.
    :var projectInfo: A ProjectInformation structure containing the destination project details..
    :var clientId: Unique identifier used by the client to track any errors.
    """
    sourceProject: TC_Project = None
    projectInfo: ProjectInformation = None
    clientId: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Core._2012_02.DataManagement import GetViewableDataResponse
from typing import List
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def getViewableData(cls, input: List[BusinessObject], configuredViewerIDs: List[str], viewerConfigKey: str) -> GetViewableDataResponse:
        """
        This operation returns the Teamcenter objects, and/or files that may be viewed for each input object.  The
        Teamcenter objects and files that are returned are based on the input object, the optional list of configured
        viewer IDs that are defined in the client, and the rules defined in the defaultViewerConfig.VIEWERCONFIG and
        <viewer config ID>.VIEWERCONFIG preferences. 
        
        - The optional list of configured viewer IDs - This is a list of IDs that uniquely identify a viewer that is
        installed/enabled in the client.  For example, in the rich client there is a viewer that is registered to the
        "MSWordViewer" ID. This viewer renders MSWord files but is only enabled on the Windows platform.  So, when
        running the rich client on Windows, "MSWordViewer" would be in the list of configured viewer IDs, but on Unix
        it would not be in the list, since MSWord isn't supported on Unix.   By giving this listing to the operation,
        only the rules that are legal for the input list of viewer IDs are processed.  If no list is provided, then all
        the rules are processed, and it's up to the client to determine how to handle the data.
        
        
        
        -  For information related to the defaultViewerConfig.VIEWERCONFIG preference, please see the My Teamcenter
        Guide or the Preferences and Environment Variables Reference.
        
        
        
        Common uses for this operation would be to retrieve viewable data for a selected object in the UI in order to
        present the viewable data in the UI.  
        
        For example, the rules may state that when an ItemRevision object is selected, and an MSWord dataset and JT
        dataset are related to that ItemRevision object, the viewable data should be the JT dataset. 
        
        Alternatively, the rules may be configured such that the ItemRevision object itself is viewable, and in that
        case, only the ItemRevision object is returned.
        
        Exceptions:
        >Thrown if the defaultViewerConfig.VIEWERCONFIG  (or the <viewer config id>.VIEWERCONFIG) preference may not be
        loaded in order to process viewable data.  These cases are:
        - 263001 - The viewer configuration preference  does not exist.  Please add it.
        - 263002 - The viewer configuration preference  has no value.  Please add one.
        - 263003 -The viewer configuration preference  is malformed.  Please correct.
        
        """
        return cls.execute_soa_method(
            method_name='getViewableData',
            library='Internal-Core',
            service_date='2012_02',
            service_name='DataManagement',
            params={'input': input, 'configuredViewerIDs': configuredViewerIDs, 'viewerConfigKey': viewerConfigKey},
            response_cls=GetViewableDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.Bom._2008_06.StructureManagement import BaselineResponse, RemoveChildrenFromParentLineResponse, AddOrUpdateChildrenToParentLineResponse, BaselineInput, AddOrUpdateChildrenToParentLineInfo
from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def removeChildrenFromParentLine(cls, bomlines: List[BOMLine]) -> RemoveChildrenFromParentLineResponse:
        """
        This operation allows developers to remove a BOMLine from an assembly /product structure. This operation takes
        vector of BOMLine business objects as input, which allows removal of multiple BOMLines from the structure in a
        single operation.
        
        Use cases:
        User wants to remove two lines. He/She invokes the operation with the lines, and the lines are removed.
        """
        return cls.execute_soa_method(
            method_name='removeChildrenFromParentLine',
            library='Bom',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'bomlines': bomlines},
            response_cls=RemoveChildrenFromParentLineResponse,
        )

    @classmethod
    def createBaseline(cls, inputs: List[BaselineInput]) -> BaselineResponse:
        """
        Creates a new Baseline ItemRevision based on a work in progress ItemRevision.  If the input ItemRevision
        consists of a PSBOMViewRevision that represents a multi level structure, all components of the structure are
        baselined in a recursive fashion. If smart baseline option is enabled at the site, then components of the
        structure will be baselined only if they satisfy the criteria set forth by smart baseline feature. Released
        ItemRevision objects are not baselined, unless the specific name of ReleaseStatus object is mentioned in the
        preference Baseline_allowed_baserev_statuses.
        """
        return cls.execute_soa_method(
            method_name='createBaseline',
            library='Bom',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'inputs': inputs},
            response_cls=BaselineResponse,
        )

    @classmethod
    def addOrUpdateChildrenToParentLine(cls, inputs: List[AddOrUpdateChildrenToParentLineInfo]) -> AddOrUpdateChildrenToParentLineResponse:
        """
        This operation takes item / item revision (depending on precise or imprecise structure) or a GDE. It takes view
        type to create a BOMView for the parent line in a product structure.  When the BOMLine for the item/item
        revision is provided and client id is empty, an update will be performed.
        
        Use cases:
        - User wants to update properties of two lines. He/She invokes the operation with the two lines and property
        values. The two lines will be updated with the specified property values.
        - User wants to create two lines with certain initial property values. He/she invokes the operation with the
        parent line, the two items to add and the initial property values. Two new lines will be created with the
        initial property values.  
        
        """
        return cls.execute_soa_method(
            method_name='addOrUpdateChildrenToParentLine',
            library='Bom',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'inputs': inputs},
            response_cls=AddOrUpdateChildrenToParentLineResponse,
        )



from __future__ import annotations

from tcsoa.gen.Visualization._2011_02.StructureManagement import CreateVisSCsFromBOMsInfo, CreateVisSCsFromBOMsResponse
from tcsoa.gen.Visualization._2013_05.DataManagement import LaunchInfoResponse, SessionInfo2, IdInfo2
from typing import List
from tcsoa.gen.Visualization._2013_05.StructureManagement import OptionKeyToOptionValueMap
from tcsoa.gen.Visualization._2011_02.DataManagement import UserAgentDataInfo, ServerInfo
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def createLaunchInfo(cls, idInfos: List[IdInfo2], serverInfo: ServerInfo, userDataAgentInfo: UserAgentDataInfo, sessionInfo: SessionInfo2) -> LaunchInfoResponse:
        """
        This service generates a VVI file which is used to launch Teamcenter Visualization viewers with selected
        objects from Teamcenter and preserve a two way communication link between the viewer and the server.  This
        operation can return the VVI type information as a string buffer or as a read file ticket to a vvi/vfz file in
        the FMS transient file volume.
        NOTE: VVI and VFZ files are not intended to be persisted and should be generated with each launch of
        visualization. For example, the VVI format is not guaranteed to be supported if the server or viewer is
        updated. VFZ files are used if more than one object is launched at a time, while VVI files are used for single
        objects.
        The client can retrieve the VVI/VFZ launch file information via string buffer or through the transient file
        volume. This is controlled by setting the hasTransientVolume flag of the SessionInfo2 input structure.  With
        the hasTransientVolume flag set to false, the launch info returns a vvi string buffer for each launch object or
        as a stream of vvi string buffers for multiple launch objects.  Using the API in this way can avoid setup and
        use of the FMS system directly by the calling client.  It is the responsibility of the client to decipher the
        response data structure.  For example, the vvi string buffer(s) can be written out as a vvi or vfz file on the
        client and passed to visualization, or the string buffer can be passed directly to embedded visualization if
        using the PLMVis toolkit.  With the hasTransientVolume flag set to true, the operation requires the Teamcenter
        File Management System (FMS) to be installed (including FCC and transient volumes) for retrieval of the VVI
        file from the transient file volume. This operation generates the launch file (VFZ or VVI), stores it in the
        FMS transient volume, and returns the FMS ticket. The client that initiated this operation is responsible for
        downloading the transient file (VVI or VFZ) with the transient ticket from transient volume to a local file
        system. The transient (VVI or VFZ) file is consumed by the Teamcenter Visualization client. The viewer will
        then establish the server connection and load the object(s) specified in the VVI file.  Launch on multiple
        objects will generate a VFZ file (zip of all the vvi files) and transient ticket of VFZ file would be sent to
        client.
        This service supports launch on Teamcenter persistent objects like Dataset, Item, ItemRevision,
        BOMViewRevision, BOMView. It also supports launch of selected BOMLines of a configured structure from Structure
        Manager or BOPLines from Manufacturing Process Planner, but in this case the caller must first create a
        VisStructureContext object. Valid launch object types and behavior such as priority order can be configured
        with the Teamcenter Preferences VMU_Datasets, VMU_FileSearchOrder and VMU_RelationSearchOrder.
        """
        return cls.execute_soa_method(
            method_name='createLaunchInfo',
            library='Visualization',
            service_date='2013_05',
            service_name='DataManagement',
            params={'idInfos': idInfos, 'serverInfo': serverInfo, 'userDataAgentInfo': userDataAgentInfo, 'sessionInfo': sessionInfo},
            response_cls=LaunchInfoResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def createVisSCsFromBOMs(cls, info: List[CreateVisSCsFromBOMsInfo], options: OptionKeyToOptionValueMap) -> CreateVisSCsFromBOMsResponse:
        """
        This operation takes a list of BOMLines (the occurrences list) and returns the VisStructureContext objects
        representing the configuration state of the BOMWindow (referred to as the configuration recipe). This
        configuration includes:
        - The occurrence UID chains for the input/selected BOMLines up to but not including the top line. 
        - Optional IMANFile reference to the PLMXML static representation of the BOMWindow.
        
        
        
        This service supports both the interoperation of selected BOMLines from the Teamcenter Rich Client to
        Teamcenter Visualization and also the capture/persistence of the configuration recipe for a particular
        BOMWindow. The occurrence list records the selected BOMLines at the time of interoperation and can be used in
        later operations to populate/expand a BOMWindow with those same occurrences.
        
        Use cases:
        When the user desires to create a persistent object that records the configuration recipe of a particular
        BOMWindow. The resulting VisStructureContext object would assumedly be used to later reconstruct a BOMWindow
        with the same configuration recipe and the recorded UID occurrence chains would be used to populate/expand the
        constructed BOMWindow with specific BOMLines. For example, this operation is used when sending selected
        BOMLines from the Structure Manager to Teamcenter Visualization and also to capture the configuration recipe
        for storage in Vis Sessions.
        
        Visualization pruned launch use case
        - User opens a structure in Structure Manager (SM)/Multi Structure Manager (MSM)/Manufacturing Process Planner
        (MPP), and configures it
        - User selects some lines they want to send to visualization as a pruned structure
        - System calls createVisSCsFromBOMs to record the selected lines and the configuration of the BOM to send
        
        
        
        Visualization session save use case
        - 1.    User performs Visualization pruned launch use case and loads occurrences into visualization
        - 2.    User creates some authored visualization content (e.g. snapshots, motions, etc)
        - 3.    User saves session to Teamcenter
        - 4.    System calls createRecipesFromBOMs operation to capture the configuration and any pruning information
        as a VisStructureContext object.  UID of object returned.
        - 5.    System writes the VisStructureContext object reference into the visualization session data
        - 6.    System saves the visualiation session dataset to Teamcenter, and relates it to the VisStructureContext
        object created by the service
        
        
        
        Visualization Technical Illustration and 3D Markup save use cases
        Similar to session save use case, except saving a different data type.  Uses this service to create the recipe
        for the authored visualization data in the Teamcenter data model.
        
        Use Case Dependencies: 
        The createVisSCsFromBOMs operation is called with input BOMLines from an existing BOM Window. Therefore, the
        BOMWindow must have already been created and populated with at least a top line.
        """
        return cls.execute_soa_method(
            method_name='createVisSCsFromBOMs',
            library='Visualization',
            service_date='2013_05',
            service_name='StructureManagement',
            params={'info': info, 'options': options},
            response_cls=CreateVisSCsFromBOMsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0Qualification, User
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AssignUserQualificationInfo(TcBaseObj):
    """
    List of  AssignUserQualificationInfo structures, each containing information required for assigning a
    Fnd0Qualification object to a Teamcenter User.
    
    
    :var user: Teamcenter User object to assign the qualification.
    :var qualification: Fnd0Qualification object to assign user qualification.
    :var levelName: Name of the Qualification level.
    :var effectiveDate: EffectiveDate for the Qualification to assign.
    :var expiryDate: Expiry Date for for the Qualification to assign.
    """
    user: User = None
    qualification: Fnd0Qualification = None
    levelName: str = ''
    effectiveDate: datetime = None
    expiryDate: datetime = None


@dataclass
class ManageQualificationInfo(TcBaseObj):
    """
    Information required to create Fnd0Qualification objects. 
    
    :var qualificationName: A unique name for the Fnd0Qualification.
    :var description: Description of the Fnd0Qualification.
    :var isExpiryDateRequired: Indicates whether effective and expiry dates are required for the Qualification. This is
    used to decide whether effective and expiry dates are required while assigning the Fnd0Quailification to a
    Teamcenter User.
    :var qualificationLevels: The levels are defined on Fnd0Qualification object as ordered list of strings. Levels
    signifies the ranking amongst the user once the Fnd0Qualification is assigned to User with a level.  Levels are
    defined by administrator. Valid values for level can be any string. For example "Advanced" , "Beginner" etc.
    """
    qualificationName: str = ''
    description: str = ''
    isExpiryDateRequired: bool = False
    qualificationLevels: List[str] = ()


@dataclass
class QualificationLevelInfo(TcBaseObj):
    """
    List of QualificationLevelInfo structures containing the Fnd0Qualification object and the level to remove.
    
    :var qualification: Fnd0Qualification object to remove qualification level.
    
    
    :var levelName: Name of Level to remove.
    """
    qualification: Fnd0Qualification = None
    levelName: str = ''


@dataclass
class RemoveUserQualificationInfo(TcBaseObj):
    """
    List of RemoveUserQualificationInfo structures, each containing the User and the Fnd0Qualification to be removed.
    
    :var user: User Object to remove qualification from the user
    :var qualification: Fnd0Qualification object to remove user qualification from the user.
    """
    user: User = None
    qualification: Fnd0Qualification = None


@dataclass
class UpdateQualificationInfo(TcBaseObj):
    """
    Information required to update Fnd0Qualification objects.
    
    :var qualification: Fnd0Qualification object to be updated.
    :var qualificationInfo: Qualification information to be updated.
    """
    qualification: Fnd0Qualification = None
    qualificationInfo: ManageQualificationInfo = None



from __future__ import annotations

from tcsoa.gen.Cad._2011_06.DataManagement import GetAllAttrMappingsResponse
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def getAllAttrMappings2(cls) -> GetAllAttrMappingsResponse:
        """
        This operation retrieves all CAD attribute mapping definitions. If a 'CadAttributeMappingDefinition' object has
        a path that includes a Generic Relationship Manager (GRM) or named reference part, the associated property
        descriptor and any attached 'ListOfValues' (LOV) objects will be returned. This operation also returns LOV
        attachments category information to be used in object based conditions.
        
        Since this operation returns existing Teamcenter attribute mappings, please reference the Teamcenter help
        section on creating attribute mappings.
        
        
        Use cases:
        User needs to have attributes set on an object in Teamcenter.
        
        For this operation, the client application uses the list of returned attribute mapping definitions to present
        the correct CAD attributes to the user that correspond to the correct Teamcenter attributes including property
        descriptor information about the Teamcenter attributes.
        
        
        Exceptions:
        >Service Exception    Thrown if any Teamcenter subsystem errors occur during the retrieval of all attribute
        mappings, finding the dataset type or finding the attribute mapping type.
        """
        return cls.execute_soa_method(
            method_name='getAllAttrMappings2',
            library='Cad',
            service_date='2011_06',
            service_name='DataManagement',
            params={},
            response_cls=GetAllAttrMappingsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetClassificationPropsResponse(TcBaseObj):
    """
    Holds the classification properties details returned by the getClassificationProps() operation.
    
    :var wsoClsPropDetailsMap: A map (string, ClsObjectDetails )of WorkspaceObject UID and the classification object
    details.
    :var clsAttrDetailsMap: A map  (int, ClsAttrDetails) of Classification attributes ID and its corresponding metadata.
    :var svcData: The Service Data.
    """
    wsoClsPropDetailsMap: WsoClsPropDetailsMap = None
    clsAttrDetailsMap: ClsAttrDetailsMap = None
    svcData: ServiceData = None


@dataclass
class ClsAttrDetails(TcBaseObj):
    """
    Structure describing classification attribute details.
    
    :var attributeId: The unique ID of the Classification attribute.
    :var attributeName: Name of the Classification attribute.
    """
    attributeId: int = 0
    attributeName: str = ''


@dataclass
class ClsAttrValueDetails(TcBaseObj):
    """
    Structure containing the attribute ID and corresponding value(s).
    
    :var attrId: The unique ID of an attribute.
    :var attrValue: List of the stored value(s) for the attrId.
    """
    attrId: int = 0
    attrValue: List[str] = ()


@dataclass
class ClsObjectDetails(TcBaseObj):
    """
    Structure representing Classification Object details.
    
    :var clsObjId: Alphanumeric ID of the Classification object, as defined for the object during its creation.
    :var clsObject: Reference of Classification object(icm0).
    :var unitSystem: Unit system of measure in which the Classification object is stored in.
    :var clsObjClassId: The unique ID of Classification class as defined for the object during its creation.
    :var clsObjClassName: Classification class Name.
    :var attributeValues: A list of ClsAttrValueDetails objects containing attribute IDs and their corresponding values
    for above classification object.
    """
    clsObjId: str = ''
    clsObject: BusinessObject = None
    unitSystem: str = ''
    clsObjClassId: str = ''
    clsObjClassName: str = ''
    attributeValues: List[ClsAttrValueDetails] = ()


@dataclass
class ObjectsGroupedByProperty(TcBaseObj):
    """
    A structure containing an internal property name and a map of objects to the property group id. It also contains a
    list of unmatched objects which don't match any group.
    
    :var internalPropertyName: The internal name of the property used for grouping.
    :var groupedObjectsMap: Map (BusinessObject, list of string) of business object and its associated grouping.
    :var unmatchedObjectList: List of unmatched business objects.
    """
    internalPropertyName: str = ''
    groupedObjectsMap: ObjectPropertyGroupingMap = None
    unmatchedObjectList: List[BusinessObject] = ()


@dataclass
class ColumnConfig(TcBaseObj):
    """
    This structure contains information for a column configuration within a client scope URI. It contains a unique
    column config id, a list of column definition information, and the operation type used to finalize the columns.
    
    :var columnConfigId: The unique identifier of the column configuration.
    :var operationType: The operation that was used to finalize the columns to be returned back.
    Supported values are:
    "Intersection", "Union" and "Configured".
    :var columns: List of column details.
    """
    columnConfigId: str = ''
    operationType: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class SaveColumnConfigData(TcBaseObj):
    """
    This structure contains column configuration details and a list of column definitions information.
    
    :var scope: The scope value is used to apply the UI configurations for that scope.  Supported values are :
    "LoginUser" - Returns the current configuration defined for the current login user. 
    Note: Other scope values "Site", "Group" and "Role" are not supported.
    :var scopeName: The name of a valid Teamcenter scope.
    For example "Group" scope, scopeName can be "dba","Engineering". 
    For "LoginUser" scope, this value should be empty.
    :var clientScopeURI: The unique name of the client scope containing configurations.
    Supported values are: 
    "Awp0SearchResults" for global search
    "fnd0Inbox" for inbox
    "fnd0alltasks" for all inbox tasks
    "fnd0mytasks" for my inbox tasks
    "fnd0surrogatescope" for surrogate inbox tasks
    "Fnd0Report" for report
    "Awp0ObjectNavigation" for object navigation
    "Awp0AdvancedSearch" for advanced search
    :var columnConfigId: The unique identifier for the column configuration.
    Ex: "searchResultsColConfig" for search location.
    :var columns: Ordered list of column information.
    """
    scope: str = ''
    scopeName: str = ''
    clientScopeURI: str = ''
    columnConfigId: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class SearchFilter(TcBaseObj):
    """
    A structure representing a string, date or numeric type of search filter.
    
    :var searchFilterType: The type of search filter. Supported values are "StringFilter", "DateFilter",
    "NumericFilter".
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var stringDisplayValue: The display value for a string filter. This field is applicable only if the
    "searchFilterType" field is set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter"
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter"
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the searchFilterType
    is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the results.
    :var startEndRange: The interval used to generate a range facets. E.g for Date range facets, +1DAY, +1WEEK, +1MONTH
    , +1YEAR are acceptable values. 
    Note: Currently this value is not being used. This is added for future.
    """
    searchFilterType: str = ''
    stringValue: str = ''
    stringDisplayValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


@dataclass
class SearchFilterField(TcBaseObj):
    """
    A structure containing details about a search filter field.
    
    :var internalName: The internal name for the search filter field.
    :var displayName: The display name for the search filter field.
    :var defaultFilterValueDisplayCount: The default number of search filter values to display within the search filter
    field.
    """
    internalName: str = ''
    displayName: str = ''
    defaultFilterValueDisplayCount: int = 0


@dataclass
class SearchInput(TcBaseObj):
    """
    A structure containing input search criteria.
    
    :var providerName: The name of the search provider. This is the RuntimeBusinessObject type name.
    :var searchCriteria: A map (string, string) used to perform the search. 
    For full text search, the key is "searchString", the value is  "*".
    For advanced search, the key can be
    "queryID" ,
    "quickSearchAttributeValue",
    "searchID",
    "typeOfSearch", "QUICK_SEARCH" or "ADVANCED_SEARCH", the value is unique identifier.
    :var startIndex: The start index to return the search results.
    :var maxToReturn: The maximum number of search results to return.If zero, no search results are returned; however,
    other information such as total results found and filters details are returned.
    :var maxToLoad: The maximum number of search results to load.
    :var searchFilterMap: A map (string, list of SearchFilter) containing the list of search filters for each search
    filter field. The key in the map is the property name that represents the filter category. It is in the format
    "TypeName.PropertyName". e.g WorkspaceObject.object_type, the value in the map is the value of the search filter.
    :var searchSortCriteria: The criteria to use to sort the results.
    :var searchFilterFieldSortType: The sorting type to use to order the search filter categories in the response.
    Supported values are: "Alphabetical","Priority".
    :var attributesToInflate: A list of attributes to inflate (extract the details).
    :var internalPropertyName: The internal name of the property used for grouping.
    """
    providerName: str = ''
    searchCriteria: StringMap = None
    startIndex: int = 0
    maxToReturn: int = 0
    maxToLoad: int = 0
    searchFilterMap: SearchFilterMap = None
    searchSortCriteria: List[SearchSortCriteria] = ()
    searchFilterFieldSortType: str = ''
    attributesToInflate: List[str] = ()
    internalPropertyName: str = ''


@dataclass
class SearchResponse(TcBaseObj):
    """
    A service response structure containing search results and column configurations.
    
    :var searchResults: List of business objects obtained after performing a search.
    :var totalFound: Total number of business objects found.
    :var totalLoaded: Total number of business objects loaded.
    :var searchFilterMap: A map (string, list of SearchFilter) containing the list of search filters for each search
    filter field based on the search results.
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var objectsGroupedByProperty: The structure containing an internal property name and a map of objects to the
    property group id. It also contains a list of unmatched objects which do not match any group.
    :var columnConfig: Effective column configuration for the client scope URI.
    :var serviceData: The service data object.
    :var endIndex:  Cursor end position for the results returned so far.
    """
    searchResults: List[BusinessObject] = ()
    totalFound: int = 0
    totalLoaded: int = 0
    searchFilterMap: SearchFilterMap = None
    searchFilterCategories: List[SearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    objectsGroupedByProperty: ObjectsGroupedByProperty = None
    columnConfig: ColumnConfig = None
    serviceData: ServiceData = None
    endIndex: int = 0


@dataclass
class SearchSortCriteria(TcBaseObj):
    """
    The criteria to use to sort the results.
    
    :var fieldName: The name of the field on which to perform the sorting.
    :var sortDirection: The direction in which the sorting needs to be performed - 'ASC' or 'DESC'.
    """
    fieldName: str = ''
    sortDirection: SortDirection = None


@dataclass
class ColumnConfigInput(TcBaseObj):
    """
    Contains input information required to retrieve UI column configurations from the Teamcenter database.
    
    :var clientName: The name of a client application, as represented by an instance of Fnd0Client in the Teamcenter
    database. This value must match the value of fnd0ClientName property. 
    For example: The client name for Active Workspace is "AWClient".
    :var hostingClientName: Specifies the name of a hosting client application, as represented by an instance of
    Fnd0Client, in the Teamcenter databases. This value must match a value of the fnd0ClientName property.
    
    For example: If client A is integrated with client B and the user can invoke client B commands from within client
    A, the input to getUiConfigs would specify client A as hosting client and client B as the client. If the caller
    wanted native commands for client A, client A would be specified as client and hosting client would be empty.
    :var clientScopeURI: The unique name of the client scope containing column configurations.
    For example: "Awp0SearchResults" is the client scope URI for search location.
    :var operationType: The operation that needs to be applied to finalize the columns to be returned back.
    Valid values are:
    "Intersection" - Gets the intersection of the columns for the types found in search results.
    "Union" - Gets all the columns for the types found in search results.
    "Configured" - Gets all the columns defined for requested scope irrespective of types passed in. If it does not
    find any configuration at the specified scope it will search up in the order of scopes User, Role, Group and Site.
    No need to pass type names if need to fetch all columns.
    :var columnsToExclude: List of columns which should be excluded from the final list being returned. The value
    provided should be in the format "TypeName.PropertyName". Both type name and property name should be internal
    values. 
    For example: ItemRevision.sequence_id, where '.' is the delimiter
    """
    clientName: str = ''
    hostingClientName: str = ''
    clientScopeURI: str = ''
    operationType: str = ''
    columnsToExclude: List[str] = ()


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client, pixel width of the column, a flag indicating if the column should be hidden and the column
    sort order.
    
    :var typeName: The business object type for the value displayed in the column. This can be any valid Teamcenter
    business object type.
    :var propertyName: The property name for the value displayed in the column.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500.
    :var columnOrder: The column order value is used to arrange the columns in order.
    :var hiddenFlag: If true, the column should be hidden on the client user interface.
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort.
    Sort priority value will be zero for columns not marked for sorting.
    :var sortDirection: How the columns are sorted. Supported values are: "Ascending" and "Descending". This value will
    be empty if the column is not marked for sorting.
    """
    typeName: str = ''
    propertyName: str = ''
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    sortDirection: str = ''


class SortDirection(Enum):
    """
    An enumeration indicating whether the sorting needs to be performed in ascending or descending order.
    """
    ASC = 'ASC'
    DESC = 'DESC'


"""
A map containing a list of propertyGroupID for each BusinessObject. For multi-valued properties, a single business object may be associated with multiple property group IDs.
"""
ObjectPropertyGroupingMap = Dict[BusinessObject, List[str]]


"""
The list of Classification attribute Ids and their descriptor.
"""
ClsAttrDetailsMap = Dict[int, ClsAttrDetails]


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap = Dict[str, List[SearchFilter]]


"""
Map from string to string.
"""
StringMap = Dict[str, str]


"""
The map of workspace object(WSO) to its classification properties.

- Primary key : Reference of the classified workspace object(WorkspaceObject) that was used to get the classification properties.
- Values : Details of classification object references associated with the classified workspace object(WorkspaceObject) referenced in the primary key.


"""
WsoClsPropDetailsMap = Dict[str, ClsObjectDetails]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, ImanFile, BusinessObject, Dataset, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FlaggedCell(TcBaseObj):
    """
    The cellids for which the occurrence information has to be determined and criteria applied.
    
    :var iCellId: The id of the spatial cell that need to be rendered.
    :var iBitMask: The bits represented by the integer indicates the position of the cursor in the
    'SpatialHierCellInfo::vCursorOptions' vector.  For example value 0x0003 indicates the first and second cursors
    should be enabled for this cell.
    If a cursor with empty search scope is enabled for a cell then all occurrences in the cell are configured.
    Otherwise the cell is configured only if it is within the scope of the cursors that are enabled for this cell.
    """
    iCellId: int = 0
    iBitMask: int = 0


@dataclass
class GetNodeBBoxIn(TcBaseObj):
    """
    The input structure that specifies the node object information for which bbox information is to be retrieved.
    
    :var sSpatialHierDataset: The UID of the spatial hierarchy dataset based on which the assembly is rendered using
    MMV technology.
    :var vPathObjs: The list of assembly nodes represented as occurrence thread paths for which the Bounding box
    extents need to be fetched.
    """
    sSpatialHierDataset: str = ''
    vPathObjs: List[OccurrenceThreadPath] = ()


@dataclass
class GetNodeBBoxResponse(TcBaseObj):
    """
    The response structure containing the Bounding Box extents for the supplied input nodes.
    
    :var vNodeBBox: A list of node bounding boxes that is parallel to the input node array. If any input assembly node
    is invalid or does not have a valid bounding box associated with it then its returned 'NodeBBox' is empty.
    :var serviceData: The 'ServiceData' through which the errors are communicated to the client.
    """
    vNodeBBox: List[NodeBBox] = ()
    serviceData: ServiceData = None


@dataclass
class GetSpatialCellsReadTicketsResponse(TcBaseObj):
    """
    The response structure contains the file information of the Jt files in the cells, occurrence information
    (transform, BOMLine) and cell information to be able to map the input cell id and scope with the occurrence
    information.
    
    :var vSpatialFileInfos: Vector containing the required file information for all unique files associated with the
    input cells.
    :var vOccurrenceInfos: Vector of occurrence information associated with the input cells.
    :var vSpatialCellOccInfos: Vector containing the index offsets into 'vOccurrenceInfos' for the input cells. This
    helps to map the input cell with its associated occurrence information and file information.
    :var serviceData: The 'ServiceData' through which the errors are communicated to the client.
    """
    vSpatialFileInfos: List[SpatialFileInfo] = ()
    vOccurrenceInfos: List[SpatialOccurrence] = ()
    vSpatialCellOccInfos: List[SpatialCellOccInfo] = ()
    serviceData: ServiceData = None


@dataclass
class IsSpatialHierarchyIn(TcBaseObj):
    """
    The input structure contains a set of dataset versions on which the existence of latest version check is to be
    performed.
    
    :var vpSpatialHierDatasetUids: A list of Fnd0SpatialHierarchy dataset uids on which the existence of latest version
    is to be checked.
    """
    vpSpatialHierDatasetUids: List[str] = ()


@dataclass
class IsSpatialHierarchyLatestResponse(TcBaseObj):
    """
    The response structure containing the verdict on the availabilty of newer dataset versions for the input
    Fnd0SpatialHierarchy dataset versions.
    
    :var vbLatestVersionFlags: A list of boolean values parallel to the input 'vpSpatialHierDatasetUids' indicating if
    a newer version of the dataset is available or not.
    :var serviceData: The 'ServiceData' through which the errors are communicated to the client.
    """
    vbLatestVersionFlags: List[bool] = ()
    serviceData: ServiceData = None


@dataclass
class MmvCursorOptions(TcBaseObj):
    """
    The options to be applied on the cursor.
    
    :var pMmvCursor: Reference to the cursor object that contains the scope information.
    """
    pMmvCursor: BusinessObject = None


@dataclass
class NodeBBox(TcBaseObj):
    """
    The bbox description for a node.
    
    :var vdBBox: An array of bounding boxes, in the form of xmin ymin zmin xmax ymax zmax. Multiple boxes can be
    returned for a path. The array length is always a product of number of boxes and 6.
    """
    vdBBox: List[float] = ()


@dataclass
class OccurrenceThreadPath(TcBaseObj):
    """
    Strcuture to hold the occurrence thread path chains.
    
    :var vsThreadUIDs: A list of of occurrence thread path chains. The 0th element pointing to the top thread.
    """
    vsThreadUIDs: List[str] = ()


@dataclass
class ReleaseSpatialHierarchyIn(TcBaseObj):
    """
    The input contains a set of Fnd0SpatialHierarchy dataset version uids for which the view locks are to be released.
    
    :var vsSpatialHierUids: The uids of the Fnd0SpatialHierarchy dataset versions for which the view locks are to be
    released.
    """
    vsSpatialHierUids: List[str] = ()


@dataclass
class SearchScope(TcBaseObj):
    """
    The SearchScope specifies the scope of a search.
    
    :var pBOMWindow: The BOMWindow context in which the occurrence thread path scope is specified.
    :var vsPathScopes: The list of occurrence thread paths that serve as scope qualifier.
    """
    pBOMWindow: BOMWindow = None
    vsPathScopes: List[OccurrenceThreadPath] = ()


@dataclass
class SpatialCellOccInfo(TcBaseObj):
    """
    The information regarding the file occupied in the given cell id.
    
    :var iCellId: The spatial cell id.
    :var viOccIdx: Index position of the spatial occurrences in 'vOccurrenceInfos' for the spatial cell id.
    """
    iCellId: int = 0
    viOccIdx: List[int] = ()


@dataclass
class SpatialFileInfo(TcBaseObj):
    """
    The information about the file contained in the spatial cell.
    
    :var sOrigFilename: The value of the original_file_name attribute on the Teamcenter file object.
    :var sFMSRTicket: The FMS  read ticket for the TcFile.
    :var pDataSet: The dataset to which the TcFile is associated. This is used if the client needs to refresh the file
    tickets.
    :var pTcFile: The pointer to the File. This is used if the client needs to refresh the file tickets.
    """
    sOrigFilename: str = ''
    sFMSRTicket: str = ''
    pDataSet: Dataset = None
    pTcFile: ImanFile = None


@dataclass
class SpatialHierCellInfo(TcBaseObj):
    """
    Structure containing the spatial hierarchy dataset and the cells in it for which theconfiguration, file
    information, read tickets are to be obtained.
    
    :var pBOMWindow: The BOMWindow of the structure that is currently being viewed.
    :var sSpatialHierDataset: The uid of the Fnd0SpatialHierarchy dataset version that is currently used for rendering.
    :var vCursorOptions: The cursor information for which the cells are to checked if they are within the its scope.
    :var vFlaggedCells: The information about the cell that need to be configured and the cursor
    information(represented as bits that index into 'vCursorOptions') for which the cells are checked for scope.
    """
    pBOMWindow: BOMWindow = None
    sSpatialHierDataset: str = ''
    vCursorOptions: List[MmvCursorOptions] = ()
    vFlaggedCells: List[FlaggedCell] = ()


@dataclass
class SpatialOccurrence(TcBaseObj):
    """
    Information about an occurrence row in the occurrence table.
    
    :var vdXForms: 16 doubles that represent the transform of this occurrence path in row major order.
    :var sOccPartKey: The uid of spatial cell index object in which the spatial occurrence is represented.
    :var occThreadPath: The occurrence thread chain of the configured occurrence.
    :var pBOMLine: The leaf BOMLine of the spatial occurrence.
    :var iFileIdx: Index position in 'vSpatialFileInfos' where file information can be found (used only if 'bHasFile'
    is true).
    :var iBitMask: The bit mask which represents the positions of cursors in 'vCursorOptions'. The spatial occurrence
    is within the scope of the cursors represented by this bit mask.
    :var bHasFile: Boolean information to indicate if the file information is returned for this spatial occurrence.
    :var sRefSetName: The reference set name as pointed by the note type UG REF SET, to be used by the viewer to decide
    which geometry to display.
    """
    vdXForms: List[float] = ()
    sOccPartKey: str = ''
    occThreadPath: OccurrenceThreadPath = None
    pBOMLine: BOMLine = None
    iFileIdx: int = 0
    iBitMask: int = 0
    bHasFile: bool = False
    sRefSetName: str = ''


@dataclass
class AcquireSpatialHierarchyIn(TcBaseObj):
    """
    The structure contains a set of  BomLines for which the latest version of the Fnd0SpatialHierarchy  dataset is to
    be acquired and locked for viewing.
    
    :var vpBOMLines: A list of BOMLines for which Fnd0SpatialHierarchy datasets are to be acquired and locked for
    viewing.
    """
    vpBOMLines: List[BOMLine] = ()


@dataclass
class AcquireSpatialHierarchyResponse(TcBaseObj):
    """
    The response structure contains the returned spatial hierarchy versions, the Fnd0SpatialTree named reference(*.mmv)
    file objects and its original file names and read tickets.  It also contains an index vector that maps the dataset
    array to the bomline array in the input structure.
    
    :var vpSpatialHierDatasets: A list of latest spatial hier dataset version acquired for the input bomlines.
    :var vpTcFiles: A list of file objects parallel to dataset version list.
    :var vsOrigFilenames: A list of file names parallel to dataset version list.
    :var vsFileTickets: A list of file tickets for the named reference (*.mmv file) parallel to dataset version list.
    :var viIndexVector: A list of indices parallel to the dataset version list to indicate the corresponding index in
    the input bomline array.
    :var serviceData: The 'ServiceData' through which the errors are communicated to the client.
    """
    vpSpatialHierDatasets: List[Dataset] = ()
    vpTcFiles: List[ImanFile] = ()
    vsOrigFilenames: List[str] = ()
    vsFileTickets: List[str] = ()
    viIndexVector: List[int] = ()
    serviceData: ServiceData = None


@dataclass
class CreateMmvCursorResponse(TcBaseObj):
    """
    The response structure containing a reference to the Fnd0MMVCursor object created.
    
    :var pMmvCursor: Cursor object that holds the current visibility scope.
    :var serviceData: The 'ServiceData' through which the errors are communicated to the client.
    """
    pMmvCursor: BusinessObject = None
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Cad._2014_10.DataManagement import GetAttrMappingsFilter, GetAttrMappingsResponse
from typing import List
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def getAttrMappings(cls, filter: List[GetAttrMappingsFilter]) -> GetAttrMappingsResponse:
        """
        This operation retrieves all CAD attribute mapping definitions or only the definitions that match the dataset
        and item type combinations included in the input filter.  CadAttributeMappingDefinition objects are returned
        with the associated business object name and property descriptor name.  The business object name and property
        descriptor name can be used to get the 'PropertyDescription' object from the client-side Meta Model,
        'ClientMetaModel'.  The 'PropertyDescription' object can then be used to access any needed property descriptor
        information, such as attached 'ListOfValues' (LOV) objects.
        
        Since this operation returns existing Teamcenter attribute mappings, please reference the Teamcenter help
        section on creating attribute mappings.
        
        
        Use cases:
        User needs to have attributes set on an object in Teamcenter.
        
        For this operation, the client application uses the list of returned attribute mapping definitions to present
        the correct CAD attributes to the user that correspond to the correct Teamcenter attributes including property
        descriptor information about the Teamcenter attributes.
        
        
        Exceptions:
        >'Teamcenter::Soa::Server::ServiceException'
        
        Service Exception    Thrown if any Teamcenter subsystem errors occur during the retrieval of all attribute
        mappings, finding the dataset type or finding the attribute mapping type. 
        """
        return cls.execute_soa_method(
            method_name='getAttrMappings',
            library='Cad',
            service_date='2014_10',
            service_name='DataManagement',
            params={'filter': filter},
            response_cls=GetAttrMappingsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.gen.Internal.StructureManagement._2016_10.Effectivity import OccEffCutbackInfo, NetEffectivityResponse, CutbackUnitEffectivityResponse
from typing import List
from tcsoa.base import TcService


class EffectivityService(TcService):

    @classmethod
    def getUnitNetOccurrenceEffectivity(cls, bomlines: List[BOMLine]) -> NetEffectivityResponse:
        """
        The operation provides the following information on a BOMLine in the given BOMWindow structure: its occurrence
        effectivity, its calculated net effectivity (which is based on its unit occurrence effectivity), and whether
        the BOMLine would configure based on this calculated net effectivity. Note it is the occurrence Effectivity
        that actually determines whether the BOMLine is configured.
        
        The net effectivity calculation is based on the intersection of a BOMLine&rsquo;s unit occurrence Effectivity
        with the unit net effectivity of its parent hierarchy. For example, given a BOMLine with a unit occurrence
        Effectivity with range of 1-10 for an end Item, a child BOMLine with unit occurrence Effectivity with a range
        of 1-100 for the same end Item, would have a unit net effectivity of 1-10 for that end Item. That is, the child
        BOMLine effectivity range is constrained by the limited range of its parent BOMLine.
        
        Use cases:
        User would like to see net effectivity information for a BOMLine within a structure.
        """
        return cls.execute_soa_method(
            method_name='getUnitNetOccurrenceEffectivity',
            library='Internal-StructureManagement',
            service_date='2016_10',
            service_name='Effectivity',
            params={'bomlines': bomlines},
            response_cls=NetEffectivityResponse,
        )

    @classmethod
    def cutbackUnitOccurrenceEffectivity(cls, occEffCutbacks: List[OccEffCutbackInfo]) -> CutbackUnitEffectivityResponse:
        """
        The operation manages unit occurrence effectivity cutback information involving  PSOccurrence children of a
        given PSBOMViewRevision. 
        
        The cutback action on unit occurrence effectivity allows a user to define PSOccurrence children within a
        specific PSBOMViewRevision which should not be effective for the same units; designating one or more
        PSOccurrence objects as "replacing" objects, and one or more sibling PSOccurrence objects as being effectively
        "replaced" by the replacing objects. With this relationship identified, the user then stipulates a desired
        effectivity for the replacing objects, and the feature automatically proposes the effectivity range adjustments
        or "cut back" that would be needed in the unit occurrence effectivity of both the replacing and the replaced
        PSOccurrence objects. 
        
        This service does not currently support the applying of the cutback proposal values to the unit occurrence
        effectivities of the identified PSOccurrence objects. The applying of the cutback proposal values is done by
        the "PS-occ-effectivity-cutback" action workflow handler.
        
        The cutback action has two steps: setup and proposal. For example, during the cutback setup, you provide
        cutback information that a PSOccurrence "PartM" with a unit occurrence Effectivity with range of 1-UP for an
        end Item is to be replaced by a sibling PSOccurrence "PartN" with unit occurrence Effectivity with a range of
        1-100 for the same end Item. In this example both PartM and PartN are effective for the units 1-100, but the
        desire is to have them be mutually exclusive. If you enter a desired unit occurrence effectivity of 50-100 for
        the end Item, the feature provides a proposal that PartN would have a unit net effectivity with range 50-100
        for the same end Item, and a proposal that PartM would have a unit net effectivity with range 1-49,101-UP for
        the same end Item.
        
        Use cases:
        View Cutbacks &ndash; you indicate the PSBOMViewRevision for which cutback information is to be obtained. The
        result is the list of related cutback objects.
        
        Setup Cutback &ndash; you indicate which PSOccurrence objects within a given PSBOMViewRevision are to be
        mutually exclusive, designating some as replacing occurrences whose unit occurrence effectivities will affect
        those on occurrences designated as replaced. You enter an effectivity range and end item to drive the cutback
        action. The result is the entered cutback information is saved, proposal effectivity ranges are generated, and
        the cutback data is returned along with proposed effectivity ranges per designated PSOccurrence.
        
        Update Cutback &ndash; you modify which PSOccurrence objects within a given PSBOMViewRevision are to be
        mutually exclusive, designating some as replacing occurrences whose unit occurrence effectivities will affect
        those on occurrences designated as replaced. You change the effectivity range and end item to drive the cutback
        action. The result is the entered cutback information is saved, proposal effectivity ranges are re-generated,
        and the cutback data is returned along with proposed effectivity ranges per designated PSOccurrence.
        
        Delete Cutback &ndash; from a  list of cutbacks for a given PSBOMViewRevision, you select an active cutback to
        delete. The result is the active cutback is deleted, along with the related proposed effectivity ranges per
        designated PSOccurrence. 
        
        Override Proposal &ndash; from a list of cutbacks for a given PSBOMViewRevision, you select an active cutback
        to view. The active cutback along with proposed effectivity ranges per designated PSOccurrence are displayed.
        You select a designated PSOccurrence and modify a proposed effectivity range. The result is the modified range
        is saved, along with re-generated proposed effectivity ranges of affected designated PSOccurrence objects.
        """
        return cls.execute_soa_method(
            method_name='cutbackUnitOccurrenceEffectivity',
            library='Internal-StructureManagement',
            service_date='2016_10',
            service_name='Effectivity',
            params={'occEffCutbacks': occEffCutbacks},
            response_cls=CutbackUnitEffectivityResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ImportExport._2010_04.L10NImportExport import TransientFileTicketStatusInfo, ObjectsInfo, ExportObjectsForTranslationResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class L10NImportExportService(TcService):

    @classmethod
    def importTranslations(cls, fileInfo: List[TransientFileTicketStatusInfo]) -> ExportObjectsForTranslationResponse:
        """
        This operation imports localized values for all the business objects specified in list of
        'TransientFileTicketStatusInfo' structure.
        """
        return cls.execute_soa_method(
            method_name='importTranslations',
            library='Internal-ImportExport',
            service_date='2010_04',
            service_name='L10NImportExport',
            params={'fileInfo': fileInfo},
            response_cls=ExportObjectsForTranslationResponse,
        )

    @classmethod
    def exportObjectsForTranslation(cls, inpuInfo: ObjectsInfo) -> ExportObjectsForTranslationResponse:
        """
        This operation exports Teamcenter business objects into XML file based on the criteria specified by
        'ObjectsInfo' structure. This operation is used to export Teamcenter business object that contains localized
        properties so that site administrator or other privileged user can translate the properties to desired locale
        and import back into database using the 'importObjectsForTranslation' operation. If the session user does not
        have the necessary privilege or if 'ObjectsInfo' structure is not valid then operation would fail and the error
        is returned in the 'ServiceData'. This operation is used to export selected business objects for review or
        translate into desired locale. These two modes are explained below:
        
        - Review: This exports business objects into XML file for selected statuses where valid localized translation
        exist for given locale. This exported file can be used by language expert to review existing translation for
        their accuracy.  
        - Translate: The operation exports business objects into XML file for which there is no translation so that
        language expert can use this XML file and provide a valid language translation in selected locale.
        
        """
        return cls.execute_soa_method(
            method_name='exportObjectsForTranslation',
            library='Internal-ImportExport',
            service_date='2010_04',
            service_name='L10NImportExport',
            params={'inpuInfo': inpuInfo},
            response_cls=ExportObjectsForTranslationResponse,
        )

    @classmethod
    def filterObjectsForTranslation(cls, inputInfo: ObjectsInfo) -> ServiceData:
        """
        This operation is very similar to 'exportObjectsForTranslation'. The only difference is this operation returns
        a list of filtered objects instead of returning an XML file. This operation can be useful to present the
        filtered list of business objects that contain localized properties to the end user as oppose to creating an
        xml files however this can't be used to import the selected objects or modify the objects. If user intention is
        to modify the localized values for given business object then it's recommended to use
        'exportObjectsForTranslation' as opposed to this operation. This operation is used to filter selected business
        objects for review or translate into desired locale. These two modes are explained below:
        - Review: This filters business objects for selected statuses where valid localized translation exist for given
        locale. 
        - Translate: The operation filters and returns business objects for which there is no translation.
        
        """
        return cls.execute_soa_method(
            method_name='filterObjectsForTranslation',
            library='Internal-ImportExport',
            service_date='2010_04',
            service_name='L10NImportExport',
            params={'inputInfo': inputInfo},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Cfg0ConfiguratorPerspective
from tcsoa.gen.Internal.Configurator._2017_11.ConfiguratorManagement import ConfigurationSessionInfoInput, GetContextBasedVariantExprsResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def getContextBasedVariantExpressions(cls, targetObjects: List[BusinessObject], perspective: Cfg0ConfiguratorPerspective) -> GetContextBasedVariantExprsResponse:
        """
        This operation returns the variant expressions associated with the given set of target objects based on the
        Configurator Perspective input which provides the Revision Rule and Configuration Context.
        
        Use cases:
        Use case 1 :
        User can get variant expressions of an existing object. For example 
        
        Create an instance of default rule (Cfg0DefaultRule) by using operation setVariantExpressions and set subject
        condition as "Color = Red". To retrieve this variant expression getContextBasedVariantExpressions   can be used
        which returns variant expression formula and the client representable structure format.
        
        Use case 2:
        If Option Value is not allocated in the input Configurator Context, only the Value text is returned in
        response, Option tag is empty. For Example,
        
        Create an instance of inclusion rule (Cfg0InclusionRule) under Context1 with following subject and
        applicability:
        Subject: Transmission = Manual (Manual is allocated to both Context1 and Context2)
        Applicability: Engine = V4 (V4 is allocated to only Context1)
        Now allocate the Inclusion Rule to Context2. Open Inclusion Rule from Context2 in Expression Editor. Option V4
        is shown as Read Only Text.
        """
        return cls.execute_soa_method(
            method_name='getContextBasedVariantExpressions',
            library='Internal-Configurator',
            service_date='2017_11',
            service_name='ConfiguratorManagement',
            params={'targetObjects': targetObjects, 'perspective': perspective},
            response_cls=GetContextBasedVariantExprsResponse,
        )

    @classmethod
    def setConfigurationSessionInfo(cls, inputs: List[ConfigurationSessionInfoInput]) -> ServiceData:
        """
        This operation attaches the configuration session information on the target objects. The information is
        extracted from the input perspective and the configuration profile parameters. The target objects must
        implement ConfigurationSession behavior. Please refer to the documentation of the business object to find out
        whether the object implements ConfigurationSession behavior. Also the input perspective should provide exactly
        one configurator context.
        
        Use cases:
        During Content Solve a user wants the Product Variability defined within the Configurator Context object to be
        honored. Call this service, pass the corresponding Cfg0ConfiguratorPerspective and Configuration Profile
        required for the Solve. This service will set this information on the target object and the same will be
        honored during Content Solve.
        """
        return cls.execute_soa_method(
            method_name='setConfigurationSessionInfo',
            library='Internal-Configurator',
            service_date='2017_11',
            service_name='ConfiguratorManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )



from tcsoa.gen.Configurator._2014_12.services import ConfiguratorManagementService as imp0
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):
    getAdmissibility = imp0.getAdmissibility



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2016_03.StructureVerification import AlignMatchedCandidateElem, AdditionalData, FindMatchingCandidatesResp, SearchCandidateElems, AlignMatchedCandidatesResp
from typing import List
from tcsoa.base import TcService


class StructureVerificationService(TcService):

    @classmethod
    def alignMatchedCandidates(cls, inputObjects: List[AlignMatchedCandidateElem], additionalInfo: AdditionalData) -> AlignMatchedCandidatesResp:
        """
        This operation aligns matched pair of BOMLine objects. Alignment means the Absolute Occurrence Identifier
        (IDInContext Top Level) will be in sync after the operation completes successfully or the Manufacturing BOMLine
        object will be replaced by the Engineering BOMLine object. In addition, the specified properties will be
        synced. The properties to be aligned will be governed by the MEAlignPropertiesList preference and any
        additional ones passed via the additionalInfo generic structure using the key: AlignProperties and the value
        being a vector of internal BOMLine object property names.
        The AdditionalData type argument currently supports the following:
        AdditionalInfo.strMap:
        Key: AlignProperties
        Value: A list of internal BOMLine property names to be synced.
        """
        return cls.execute_soa_method(
            method_name='alignMatchedCandidates',
            library='Internal-StructureManagement',
            service_date='2016_03',
            service_name='StructureVerification',
            params={'inputObjects': inputObjects, 'additionalInfo': additionalInfo},
            response_cls=AlignMatchedCandidatesResp,
        )

    @classmethod
    def findMatchingCandidates(cls, inputObjects: List[SearchCandidateElems], additionalInfo: AdditionalData) -> FindMatchingCandidatesResp:
        """
        This operation returns candidate matching target BOMLine objects for the given source BOMLine objects, based on
        specified search criteria. The additionalInfo parameter currently supports the following:
        AdditionalInfo.strMap:
        Key: SearchCriteriaPreference
        Value: name of the preference containing the search criteria parameters at index 0.
        """
        return cls.execute_soa_method(
            method_name='findMatchingCandidates',
            library='Internal-StructureManagement',
            service_date='2016_03',
            service_name='StructureVerification',
            params={'inputObjects': inputObjects, 'additionalInfo': additionalInfo},
            response_cls=FindMatchingCandidatesResp,
        )



from __future__ import annotations

from tcsoa.gen.Ai._2006_03.Ai import ApplicationRef
from typing import List
from tcsoa.gen.BusinessObjects import RequestObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateScopedMultipleStructureResponse(TcBaseObj):
    """
    GenerateScopedMultipleStructureResponse struct
    
    :var ticket: The transient file ticket to be used for downloading the generated plmxml
    :var data: partial failures are returned - along with object ids for each plmxml data could not be generated.
    """
    ticket: str = ''
    data: ServiceData = None


@dataclass
class GenerateScopedSyncRequestResponse(TcBaseObj):
    """
    GenerateScopedSyncRequestResponse struct
    
    :var request: request
    :var data: partial failures are returned - object ids for each sync data could not be created.
    """
    request: RequestObject = None
    data: ServiceData = None


@dataclass
class ObjectsWithConfig(TcBaseObj):
    """
    structure to specify multiple objects with each set potentially having it's own configuration
    
    :var apprefs: vector of ApplicationRefs specifying the objects to be included in the plmxml generation
    :var config: configuration to be used for the above set of objects.
    """
    apprefs: List[ApplicationRef] = ()
    config: Configuration2 = None


@dataclass
class Configuration2(TcBaseObj):
    """
    Configuration structure.
    
    :var useDefaultRevisionRule: if true - the Teamcenter preferences are used to pick up the rev rule. Overrides
    everything if present.
    :var revRuleName: if id is NULLTAG, then used to specify the revisionrule by name.
    :var variantRule: ApplicationRef of a variantrule - only needed if revrule is being specified elsewhere.
    :var configuringObject: revisionrule object or structurecontext
    :var relatedContexts: vector of ApplicationRefs specifying the structure contexts to help create the linked windows
    """
    useDefaultRevisionRule: bool = False
    revRuleName: str = ''
    variantRule: ApplicationRef = None
    configuringObject: ApplicationRef = None
    relatedContexts: List[ApplicationRef] = ()
